/* A simple non-recursive LOGIC program */

program(0, [X,Y]) :- should_take(X,Y).

  should_take(A,B) :- 
      complains_of(A,C),
      suppresses(B,C),
      suitable(B,A).

  should_take(B,A) :- does_not_aggravate(A,C),suffers_from(B,C).

  suppresses(aspirin,pain).
  suppresses(lomotil,diarrhea).

  does_not_aggravate(aspirin,impaired_liver_function).
  does_not_aggravate(lomotil,peptic_ulcer).

  complains_of(john,pain).
  complains_of(sue,diarrhea).

  suffers_from(john,impaired_liver_function).
  suffers_from(sue,peptic_ulcer).

/*  TEST SET 1 

?-   mixd(should_take(sue,X)).
?-   mixd(should_take(X,Y)).
?-   mixb(should_take(X,Y)).
*/

/*   DATA FOR VARIOUS TESTS */

    data(a).  data(b).  data(c).

    p(a) :- write(a).
    p(b) :- write(b).
    p(X) :- write(ok).

    int(1).
    int(X) :- int(X1), X is X+1.

/*  WITH SIDE EFFECT PREDICATES */

    program(1, X) :- write(X), data(X).

/*  TEST SET 2 

?-     mixd(program(1, X)).
?-     mixd(program(1, a)).
?-     mixd(data(X)).
*/

/*   WITH META-PREDICATES */

    program(2, X) :- var(X), data(X).
    program(3, X) :- X.

/*    TEST SET 3

?-     mixd(program(2, X)).
?-     mixd(program(3, X)).
*/ 

/*    CUTS  - Evaluable  */

    program(4, X) :- data(X),!.
    program(4, X) :- p(X).

/*  CUTS -- Not evaluable */

    program(5, X) :- p(X),!.
    program(5, X) :- data(X).

/*  CUTS -- backward propagation test */

    program(6, X) :- l(X), !, data(X).

/*   CUTS -- Not Evaluable and deeply hidden */
/*   Note l(X) is left undefined on purpose */


    program(7, X) :- prog71(X), prog72(Y).

    prog71(X) :- prog(X), q(X).
    prog72(X) :- data(X), !, int(1),!.
    prog(X) :- l(Q), !.

    q(a).
    q(b).
    q(c) :- l(Q), !, p(X).
    q(d).

/*  TEST ORS */

    or_test(1,X) :- ((X=1, (X=2; ((X=1,!); (X=1, p(X))); (X=1; z(X))))).
    or_test(2,X) :- ((X=1, (X=2; (X=1; (X=1, p(X))); (X=1; z(X))))).
    or_test(3,X) :- ((X=1, (X=2; (X=1; (X=1, !, p(X))); (X=1; z(X))))).
    or_test(4,X) :- ((X=1, (X=2; (X=1,!,z(X))))).
    or_test(5,X) :- (fail; X=1,!,z(X)).
    or_test(6,X) :- (fail;fail).

/*  IF_TESTS */

    if_test(1,X) :- true -> X=2; X=3.
    if_test(2,X) :- l(Q) -> X=2; X=3.
    if_test(3,X) :- fail -> X=2; X=3.
    if_test(4,X) :- data(X), (true -> Y=2;Y=3).
    if_test(5,X) :- data(X), (true -> Y=2,!;Y=3).
    if_test(6,X) :- data(X), (true -> Y=2;Y=3,!).
    if_test(7,X) :- data(X), (fail -> Y=2;Y=3,!).
    if_test(8,X) :- data(X), (fail -> Y=2;Y=3),!.
    if_test(9,X) :- data(X), (l(Q) -> Y=2;Y=3).
    if_test(10,X) :- data(X), (l(Q),! -> Y=2;Y=3).
    if_test(11,X) :- data(X), (l(Q) -> Y=2,!;Y=3).
    if_test(12,X) :- data(X), (l(Q) -> Y=2;Y=3,!).
    if_test(13,X) :- (fail -> fail;true), X=1.
    if_test(14,X) :- (fail -> true;fail), X=1.
    if_test(15,X) :- (true -> true;fail), X=1.


/*  SET TESTS  */

    set_test(1, Y) :-    bagof(X, (X=1;X=2;X=3), Y).
    set_test(2, Y) :-    bagof(X, (X=1; l(Q);X=3), Y).
    set_test(3, Y) :-    bagof(X, Y^((Y=1;Y=2),(X=1;X=2;X=3)), Y).

