
/*
   The "clause selection" meta-interpreter from:
      @inproceedings{Owen,
	  AUTHOR= {Owen, S.},
	  TITLE= {Issues in the Partial Evaluation of Meta-Interpreters},
	  BOOKTITLE= {Proceedings {Meta88}: Workshop on Meta-programming in
		       Logic Programming, Bristol},
	  MONTH= Jun,
	  PUBLISHER={MIT Press},
	  YEAR= 1988,
	  PAGES={319-339}
	}

   February 22, 1990

*/

/* 1 */ solve(_Theory, true, _Control).
/* 2 */	solve(Theory, not Goal, Control) :-
	    not solve(Theory, Goal, Control).
/* 3 */	solve(_Theory, Goal, _Control) :-
	    system(Goal),
	    call(Goal).
/* 4 */	solve(Theory, (Goal,Rest), Control) :-
	    solve(Theory, Goal, Control),
	    solve(Theory, Rest, Control).
/* 5 */	solve(Theory, Goal, Control) :-
	    set_control(Goal, Control, NewControl),
	    select_clause(Theory, Goal, NewControl, SubGoals),
	    solve(Theory, SubGoals, NewControl).

/* 6 */	select_clause(Theory, Goal, Control, SubGoals) :-
	    member(Theory, (Goal:-SubGoals), Id),
	    check_global_constraints(Theory, Goal, Id),
	    check_local_constraints(Theory, Goal, Id).

/* 7 */	set_control(q(X,Y), _Control, sameSheet) :-
	    ground(X),
	    ground(Y),
	    inSameSheet(X, Y),
	    !.
/* 8 */ set_control(_Goal, C, C).


check_global_constraints(rules, Control, Id) :-
	Control \= sameSheet,
	!.
check_global_constraints(rules, C, Id) :-
	Id \= 80,
	Id \= 82.

/* The clauses are numbered as in the original paper */
