
/*
   The "goal ordering" meta-interpreter from:
      @inproceedings{Owen,
	  AUTHOR= {Owen, S.},
	  TITLE= {Issues in the Partial Evaluation of Meta-Interpreters},
	  BOOKTITLE= {Proceedings {Meta88}: Workshop on Meta-programming in
		       Logic Programming, Bristol},
	  MONTH= Jun,
	  PUBLISHER={MIT Press},
	  YEAR= 1988,
	  PAGES={319-339}
	}

   February 22, 1990

*/

/* 1 */ solve(Theory, true).
/* 2 */ solve(Theory, Goals) :-
	    Goals \= true,
	    select(Theory, Goals, Goal, Left, Right, Flag),
	    reduce(Flag, Theory, Goal, SubGoals),
	    combine(SubGoals, Left, Right, NewGoals),
	    solve(Theory, NewGoals).

/* 3 */ reduce(expand, Theory, Goal, SubGoals) :-
	    member(Theory, (Goal:-SubGoals)).
/* 4 */ reduce(solve, Theory, Goal, true) :-
	    member(Theory, (Goal:-SubGoals)),
	    solve(Theory, SubGoals).
/* 5 */ reduce(_, Theory, not Goal, true) :-
	    notsolve(Theory, SubGoals).
/* 6 */ reduce(_, Theory, Goal, true) :-
	    system(Goal),
	    call(Goal).

/* 7 */ select(_, Goal, Goal, true, true, solve) :-
	    Goal \= (_,_).
/* 8 */ select(Theory, Goals, Goal, true, Rest, solve) :-
	    uniqs(Theory, Goals, Uniqs),
	    Uniqs \= true,
	    least_cost(Theory, Uniqs, Goal),
	    remove(Goal, Goals, Rest),
	    !.
/* 9 */ select(Theory, Goals, Goal, Left, Right, expand) :-
	    split_at_member(Goal, Goals, Left, Right),
	    uniquely_matchable(Theory, Goal),
	    !.
/* 10 */ select(_, Goals, Goal, true, Rest, solve).

/* The clauses are numbered as in the original paper */
