/*

  This program was sent to me (Arun Lakhotia) by Ronen Feldman by
  e-mail for the purpose of experimenting with ProMiX.

    Date: Mon, 2 Jul 90 11:54:41 EDT
    From: feldman@cs.cornell.edu (Ronen Feldman)

  The program has been edited -- moved predicate definitions around,
  made one predicate per line, etc. for the sake of clarity of
  presentation. In the process the interpreter and data part of the
  code has also become explicit

   Arun Lakhotia -- July 2, 1990

*/
%---tower.dom
%---query is: tower(T,20,30,0,0,S) - T is top block, S is the state
%---General query: tower(T,MinY,MaxY,MinX,MaxX,S)
%--answer is: 
%T = d
%S = do(transfer(d,c),do(transfer(c,a),do(transfer(a,b),s0))) 


% INTERPRETER

tower(Top,Ymin,Ymax,Xmin,Xmax,S) :-
	astate(S),
        clear(Top,S),
        xpos(Top,X,S),
	ypos(Top,Y,S),
        inborders(X, Y, Xmin, Xmax, Ymax, Ymin).

inborders(X, Y, Xmin, Xmax, Ymax, Ymin) :-
	X >= Xmin,X =< Xmax,
	Y >= Ymin,Y =< Ymax.

astate(S) :- state(S).
astate(do(transfer(X,Y),S)) :-
	astate(S),
        liftable(X,S),
        free_space(Y,S),
        X \== Y.

clear(Z,do(transfer(X,Y),S)) :-
	on(X,Z,S),
	block(Z),
	Z \== Y.
clear(A,S) :-
	fact_clear(A,S).
clear(A,do(transfer(X,Y),S)) :-
	clear(A,S),A \== Y.

on(X,Y,do(transfer(X,Y),S)) :-
	astate(do(transfer(X,Y),S)).
on(A,B,do(transfer(X,Y),S)) :-
	on(A,B,S),A \== X.

xpos(X,Pos,do(transfer(X,Y),S)) :-
	xpos(Y,Pos,S).
xpos(A,Pos,do(transfer(X,Y),S)) :-
	xpos(A,Pos,S).
xpos(X,Pos,S) :-
	fact_xpos(X,Pos,S).

ypos(X,Pos2,do(transfer(X,Y),S)) :-
	ypos(Y,Pos,S),
	height(X,H),
	X \== Y,
	Pos2 is H + Pos.
ypos(A,Pos,do(transfer(X,Y),S)) :-
	ypos(A,Pos,S).
ypos(A,Pos,S) :-
	fact_ypos(A,Pos,S).

free_space(X,S) :-
	clear(X,S),
	flat_top(X).

liftable(X,S) :- 
	clear(X,S),
        block(X).

flat_top(X) :- table(X).
flat_top(X) :- box(X).

%  DATA 

state(s0).

box(a).  box(b).  box(c).  box(d).  box(e).

block(a).  block(b).  block(c).  block(d).  block(e).

table(t).

height(a,5).  height(b,7).  height(c,5).  height(d,5).  height(e,5).

fact_clear(a,s0).  fact_clear(b,s0).  fact_clear(c,s0).
fact_clear(d,s0).  fact_clear(e,s0).

fact_ypos(a,5,s0).  fact_ypos(b,7,s0).  fact_ypos(c,5,s0).
fact_ypos(d,5,s0).  fact_ypos(e,5,s0).  fact_ypos(t,0,s0).
fact_xpos(t,0,s0).  fact_xpos(a,0,s0).  fact_xpos(b,0,s0).
fact_xpos(c,0,s0).  fact_xpos(d,0,s0).  fact_xpos(e,0,s0).

on(b,t,s0).  on(a,t,s0).  on(c,t,s0).  on(d,t,s0).  on(e,t,s0).
