%  solve(Goal, Allsolutions).
%     gives all solutions for Goal
%     leaves Goal unchanged

solve(Goal, Allsolutions) :-
	allclauses(Goal, Allclauses),
	solve_all_clauses(Allclauses, Goal, Allsolutions).

% solve_all_clauses(Allclauses, Goal, Allsolutions).
%    Allsolutions is a list of terms that unify with Goal

solve_all_clauses([], Goal, []).
solve_all_clauses([C|Cs], Goal, Allsolutions) :-
	solve_a_clause(C, Goal, R),
	solve_all_clauses(Cs, Goal, Rs),
	merge_clause_solutions(R, Rs, Allsolutions).

solve_a_clause((Head:-Body), Goal, Solutions) :-
	unifiable(Head, Goal),
	solve_body(Body, Goal, AllBodySolutions),
	transfer_solution_up(AllBodySolutions, Goal, Body, Solutions).
solve_a_clause((Head:-Body), Goal, []) :-
	not unifiable(Head, Goal),

solve_body(true, [true]).
solve_body((A,B), SolutionsB) :-
	solve_body(A, SolutionsA),
	solve_conjunction(SolutionsA, B, SolutionsB).
solve_body(A, Solutions) :-
	system(A),
        all_system_solutions(A, AllSystemSolutions),
	transfer_solution_up(AllSystemSolutions, Goal, A, Solutions).
solve_body(A, Solutions) :-
	solve(A, AllsolutionsforBodyGoal),
	transfer_solution_up(AllsolutionsforBodyGoal, A, Solutions).

% transfer_solution_up(AllsolutionsforBodyGoal, Goal, BodyGoal, Solutions) 
transfer_solution_up([], Goal, BodyGoal, []).
transfer_solution_up([B|Bs], Goal, BodyGoal, [S|Ss]) -:
	copy((Goal,BodyGoal), (S,B)),
	transfer_solution_up(Bs, Goal, BodyGoal, Ss).

%  allclauses(Goal, Allclauses)
%     gives a list of clauses whose head unify with
%     Goal. Does not do the unification
        bagof((Goal:-Body), clause(Goal, Body), Allclauses).
