:- dynamic(single_solution/1).

%%  bagOf(+Term, +Goal, -BagOfSolutions)

bagOf(Term, Call, Bag) :-
   bagOfd(Term, Call, Bag-[]).

%%  bagOfd(+Term, +Goal, -BagOfSolutions)
%%     differs from bagOf/3 as BagOfSolutions returned is a
%%     difference list, Head-Tail

    % Any Term is a solution of true
bagOfd(Term, true, [Term|Bag]-Bag) :-!.
bagOfd(Term, (Goal,Conjunction), Bag-List) :- !,
    % Hard to explain, leave it for the reader to figure out
        bagOfd((Term,Conjunction), Goal, Pairs-[]),
	bagOf_conjunct_d(Pairs, Bag-List). 
bagOfd(Term, \+Goal, Bag-Terms) :- !,
	bagOf_single_solution(Term, \+Goal, Bag-Terms).
bagOfd(Term, Goal, Bag-Terms) :-
	sys(Goal),!,
	bagOf_system_predicate(Term, Goal, SystemBag),
	make_diff_list(SystemBag, Bag-Terms).
%bagOfd(Term, Goal, Bag-Terms) :-
%	single_solution(Goal),!,
%	bagOf_single_solution(Term, Goal, Bag-Terms).
bagOfd(Term, Goal, Bag-Terms) :- 
      % get the list of clauses whose heads match goal
        all_clauses(Goal, ClauseList),
      % solve bodies and get bag of resulting heads
        bagOf_clauses_d(ClauseList, SolutionHeads-[]),!,
	term_instances_d(Term, Goal, SolutionHeads, Bag-Terms). 
bagOfd(_Term, _Goal, Bag-Bag).

        bagOf_conjunct_d([], Bag-Bag).
        bagOf_conjunct_d([(Term,Conjunction)|Pairs], List1-List2) :- 
             bagOfd(Term, Conjunction, List1-List1d),
             bagOf_conjunct_d(Pairs, List1d-List2).

        bagOf_clauses_d([], Bag-Bag).
        bagOf_clauses_d([(Head :-Body)|Pairs], List1-List2) :- 
             bagOfd(Head, Body, List1-List1d),
             bagOf_clauses_d(Pairs, List1d-List2).

bagOf_system_predicate(Term, Goal, Terms) :-
 	bagof(Term, Goal, Terms),!.
bagOf_system_predicate(_Term, _Goal, []).

bagOf_single_solution(Term, Goal, [NewTerm|Terms]-Terms) :-
	copy((Goal,Term), (NewGoal,NewTerm)),
	call(NewGoal),!.
bagOf_single_solution(_Term, _Goal, Bag-Bag).

term_instances_d(_, _,  [], Tail-Tail).
term_instances_d(Term, Goal, [Solution|Rest], [R|H]-T) :- 
        term_instance(Term, Goal, Solution, R), !,
        term_instances_d(Term, Goal, Rest, H-T). 
term_instances_d(Term, Goal, [_Solution|Rest], H-T) :- 
       % not term_instance(Term, Goal, _Solution, R), 
        term_instances_d(Term, Goal, Rest, H-T). 

term_instance(Term, Goal, Solved_Goal, CopyTerm) :- 
        copy((Term,Goal), (CopyTerm,CopyGoal)),
	unify(CopyGoal, Solved_Goal).

