%
% read a line as a list of atoms
%	bammi@mandrill.CWRU.edu
%
readl(List) :-
	read_line(String),	% read a String
	atomize(String, List). % split String into a list of atoms

atomize([], []) :-!.
atomize(String, [First|Tail]) :-
	firstAtom(String, First, RestOfString), % Pick out the First Atom
	atomize(RestOfString, Tail).		   % Pick rest
firstAtom(String, Atom, RestString) :-
	firstAtomX(String, L, RestString),
	makeAtom(L, Atom).

makeAtom([H|T], Atom) :-
	isdigit(H), !,
	atoi([H|T], Atom).
makeAtom(L, Atom) :-
	name(Atom, L).

atoi(L, Atom) :-
	atoix( L, 0, Atom).

atoix([], N, N) :- !.
atoix([H|T], N, R) :-
	R1 is ((H - 48)  + (N*10)),
	atoix(T, R1, R).

firstAtomX([H|T], [H], T) :-
	singleCharAtom(H),!.
firstAtomX([H|T], [AtomChar|Chars], Rest) :-
	atomChar(H, AtomChar), !,		% H is a valid AtomChar
	moreAtomChars(T, Chars, Rest).
firstAtomX([H|T], L, Rest) :-
	firstAtomX(T, L, Rest).		% H is not an atom char, skip it

moreAtomChars([H|T], [AtomChar|Chars], Rest) :-
	atomChar(H, AtomChar), !,
	moreAtomChars(T, Chars, Rest).
moreAtomChars(L, [], L).

isdigit(X) :- X>47, X<58.

atomChar(C, C) :- C>96, C<123.			% a .. z
atomChar(C, L) :- C>64, C<91, L is C+32.	% A .. Z
atomChar(C, C) :- C>47, C<58.			% 0 .. 9
atomChar(39, 39).				% '
atomChar(95, 95).				% _

singleCharAtom(44).
singleCharAtom(59).
singleCharAtom(58).
singleCharAtom(63).
singleCharAtom(33).
singleCharAtom(46).
singleCharAtom(43).
singleCharAtom(45).
singleCharAtom(42).
singleCharAtom(47).
singleCharAtom(94).
singleCharAtom(40).
singleCharAtom(41).
