/*  Copyright (C) 1989  Arun Lakhotia    */
/* File: accumulator.pro
*  Author: Arun Lakhotia
*  $Log:	acc.pro,v $
Revision 1.1  90/07/05  17:30:12  arun
New Release
      */

/*  INTERPRETER ENCODING ACCUMULATOR TECHNIQUE */

acc(Goal, Result) :-
     init_acc(Init),
     acc(Goal, Init, Result).


acc((GoalA,GoalB), In, Out) :- !,
     acc(GoalA, In, Mid1),
     acc(GoalB, Mid1, Mid2),
     comp_after_conjunct((GoalA,GoalB), Mid2, Out).
acc(Goal, In, Out) :-
     evaluate_direct(Goal), !,
     call(Goal),
     comp_after_direct(Goal, In, Out).
acc(Head, In, Out) :-
     unfold_clause(Head, Body),
     comp_at_head(Head, In, Mid1),
     acc(Body, Mid1, Mid2),
     comp_after_atomic(Head, Mid2, Out).


    /* SUPPORT PREDICATES */

% evaluate_direct(+GOAL) <-
%    succeeds if the GOAL should be directly evaluated instead of by
%    the technique interpreter. This provides a mechanism for
%    excluding user-defined goals and system primitives from
%    participating in the computation performed by a technique.

 /* ABSTRACT PREDICATES USED BY THE TECHNIQUE */

% init_acc(-INIT) <-
%    initializes the accumulator to INIT

% comp_at_head(+HEAD, +IN, -OUT) <-
%    computation performed immediately after matching the HEAD of a
%    clause. IN is the in-coming accumulator and OUT is the
%    accumulator after the computation.

% comp_after_conjunct(+GOALSAB, +IN, -OUT) <-
%    computation after conjunctive GOALSAB. IN is the result of
%    returned after evaluating the second conjunct. OUT is the new
%    accumulator. 

% comp_after_direct(+GOAL, +IN, +OUT) <-
%    computation after evaluating a GOAL directly by the Prolog
%    engine. IN is the in-coming accumulator and OUT is the
%    accumulator after the computation.

% comp_after_atomic(+GOAL, +IN, -OUT) <-
%    computation after evaluating a user-defined atomic GOAL. IN is
%    the in-coming accumulator and OUT is the accumulator after the
%    computation.

