/* Prototype Expert System for patient dtat management of hemodynamic
        parameters.         Author: Leon Sterling
                            Date:  June 9, 1986
*/

hd :- system_prompt,
      get_patient(Patient),
      get_hemodynamic_parameters(Patient),
      serve_consult(hem_analysis(Patient)).

hem_analysis(Patient) :-
    treatment_strategy(Patient,Strategy),
    drug(Strategy,Drug),
    dosage(Drug,Patient,Dosage),
    administer(Drug,Patient,Dosage).

/*  treatment_strategy(Patient,Strategy) <-
       This is the central part of the program which will interact with the
       model and decide what strategy should be followed.
       In the prototype there are four possible strategies:
          Using an inotropic agent
          Reducing preload
          Reducing afterload
          Adding fluids                               
*/

drug(use_inotropic_agent,dobutamine).
drug(use_inotropic_agent,dopamine).                                                                              
drug(use_inotropic_agent,levesphed).
drug(use_inotropic_agent,epinephrine).
                                    
drug(reduce_afterload,nitroprusside).

drug(reduce_preload,intravenous_nitroglycerine).

drug(add_fluids,Fluid) :-
    possible_fluid(Fluid).

possible_fluid(blood).            possible_fluid(saline).
possible_fluid(plasma).           possible_fluid(synthetic_colloid).

/* Drug Dosages  */

dosage(nitroprusside,Patient,(15,'mg. per min')).

dosage(intravenous_nitroglycerine,Patient,(0.2,'mg. initially')).

dosage(Fluid,Patient,Fluid) :-
    possible_fluid(Fluid).

/* Drug administration  */

administer(blood,Patient,X) :-
    nl, writeln(['Give ',Patient,' extra blood.']).
            
/*  Controlling input   */

get_patient(Patient) :-
    nl, write('Please enter patient name: '), read(Patient).

get_hemodynamic_parameters(Patient) :-
    is_prompted(heart_rate(Patient,HR),[]),
    is_prompted(blood_pressure(Patient,BP),[]),
    is_prompted(central_venous_pressure(Patient,CVP),[]),
    is_prompted(mean_systemic_arterial_pressure(Patient,MAP),[]),
    is_prompted(pulmonary_wedge_pressure(Patient,PWP),[]),
    is_prompted(cardiac_output(Patient,CO),[]).

systemic_vascular_resistance(Patient,SVR) :-
    mean_systemic_arterial_pressure(Patient,MAP),
    central_venous_pressure(Patient,CVP),
    cardiac_output(Patient,CO),
    SVR is (MAP-CVP)/CO * 79.92.
      
    
/*  Controlling output  */

system_prompt :-
     nl, write('Welcome to the hemodynamic patient data mangement system'), nl.

/*  Temporary hacks  */
treatment_strategy(patient1,add_fluids).

treatment_strategy(Patient,add_fluids) :-
    pulmonary_wedge_pressure(Patient,PWP),
    PWP < 10,
    typical_low_pressure_case(Patient).

  
typical_low_pressure_case(Patient) :-
    heart_rate(Patient,HR), HR > 100,
    central_venous_pressure(Patient,CVP), CVP < 5,
    systemic_vascular_resistance(Patient,SVR), SVR > 1200,
    stroke_volume(Patient,SV), SV <70.

stroke_volume(Patient,SV) :-
    cardiac_output(Patient,CO),  heart_rate(Patient,HR),SV is CO/HR.
                                
    
