
psolve(A,R) :- 
   psolve_prim(A,R).
psolve((A,B),U-W) :- psolve1(A,U-V), psolve1(B,V-W).
psolve(A,R) :- cl(A,B), psolve1(B,R).

% Cl selectively retreives clauses. It is not supposed to be
% invoked for conjunctions and built in predicates.

cl(A,B) :- 
   \+conjunction(A),
   \+built_in(A),
   unfold_clause(A,B).

psolve1((A,B),X-Z) :- psolve2(A,X-Y),psolve1(B,Y-Z).
psolve1(A,R) :- \+conjunction(A), psolve2(A,R).

conjunction((_A,_B)).

psolve2(A,R) :- 
    psolve_prim(A,R).
psolve2(A,R) :- 
    \+(A = true),
    \+built_in(A),
    psolve3(A,R).

psolve3(A,R) :- expandable(A), psolve(A,R).
psolve3(A,[A|Z]-Z) :- residual(A).

expandable(A) :- \+suspend(A).
residual(A) :- suspend(A).

psolve_prim(true,Z-Z) :- !.

psolve_prim(A is B,Z-Z) :- ground(B), !, call(A is B).
psolve_prim(A is B,[A is B|Z]-Z) :- !.

psolve_prim(A =:= B,Z-Z) :- ground(B), !, call(A =:= B).
psolve_prim(A =:= B,[A =:= B|Z]-Z) :- !.

psolve_prim(A =\= B,Z-Z) :- ground(B), !, call(A =\= B).
psolve_prim(A =\= B,[A =\= B|Z]-Z) :- !.

psolve_prim(A < B,Z-Z) :- ground(B), !, call(A < B).
psolve_prim(A < B,[A < B|Z]-Z) :- !.

psolve_prim(A =< B,Z-Z) :- ground(B), !, call(A =< B).
psolve_prim(A =< B,[A =< B|Z]-Z) :- !.

psolve_prim(A > B,Z-Z) :- ground(B), !, call(A > B).
psolve_prim(A > B,[A > B|Z]-Z) :- !.

psolve_prim(A >= B,Z-Z) :- ground(B), !, call(A >= B).
psolve_prim(A >= B,[A >= B|Z]-Z) :- !.

psolve_prim(call(A),Z-Z) :- ground(A), !, call(A).
psolve_prim(call(A),[A|Z]-Z) :- !.
