
/*
     A program to demo the working of the partial evaluator 
     "psolve" 

     See the README file for further details.

     Created by: Raghavan Srinivas and Jagadeesh Nandigam. 
     
     MODIFICATION HISTORY:

     April 27, 1990: Arun Lakhotia
        Cleaned up code to have a general interface for all the three
        demos

    NOTE: The "psdm" prefix is added to ensure that the predicates do
        not conflict with other existing predicates. While one can
        choose to expand it the way one feel likes, we suggest it be
        expanded to "PSolve DeMo"

*/

/*    DECLARATIONS OF DATA USED FOR DEMO. THIS INCLUDES THE VARIOUS
      FILES AND EXAMPLE GOALS.
*/

psdm_version(fujita, [
      'psolve/Fujita/psolve.pro',
      'psolve/Fujita/psolve_driver.pro'
       ]).
psdm_version(clean, [
       'psolve/clean/psolve.pro',
       'psolve/Fujita/psolve_driver.pro'
      ]).

psdm_version_meta_k(fujita, [
       'psolve/Fujita/meta_k.pro'
      ]).
psdm_version_meta_k(clean, [
       'psolve/clean/meta_k.pro'
      ]).

psdm_support_files([
      'psolve/support/ground.pro',
      'psolve/support/diff_to_conj.pro'
   ]).

psdm_data_files([
       'psolve/data/append.pro',
       'psolve/data/count.pro'
      ]).

psdm_goal_to_compile(1, count(append(_X,_Y,_Z),_C)).

psdm_int_to_specialize(1, 'psolve/data/count.pro').


/*    RUN DEMO TO SPECIALIZE INTERPRETER USING THE PSOLVE/COUNT
      AND THE SPECIALIZED PSOLVE
*/

psdm_demo(Version) :-
    psdm_demo(Version, _X),
    fail.

psdm_demo(Version, TestNo) :-
    psdm_load_demo(Version),!,    % don't load again
    psdm_run_demo(TestNo).

psdm_load_demo(Version) :-
    psdm_version(Version, VFiles),
    psdm_support_files(SFiles),
    psdm_data_files(DFiles),
    consult(VFiles),
    consult(SFiles),
    consult(DFiles).

psdm_run_demo(TestNo) :-
    psdm_goal_to_compile(TestNo, Goal),
    nl,nl,
    write('% Goal being compiled '), write(Goal), nl,
    psolve_all(Goal).

/*   CODE TO SPECIALIZE PSOLVE WITH RESPECT TO AN INTERPRETER.
     THE SPECIALIZATION IS DONE USING PROMIX
*/

specialize_psolve(Version) :-
    specialize_psolve(Version, _IntNo),
    fail.

specialize_psolve(Version, IntNo) :-
    load_for_promix(Version),
    set_value(loop_check_type, off),
    set_value(rewrite, off),
    !,
    specialize_interpreter(IntNo).

specialize_interpreter(IntNo) :-
    psdm_int_to_specialize(IntNo, IFiles),
    nl,nl, write('% Interpreter being specialized: '), write(IFiles), nl,
    mixf(psolve, IFiles),
    !.  % Donot look for more IFiles in the path

load_for_promix(Version) :-
    psdm_version(Version, VFiles),
    psdm_support_files(SFiles),
    psdm_data_files(DFiles),
    rdf(VFiles),
    rdf(SFiles),
    rdf(DFiles),
    psdm_version_meta_k(Version, MFiles),
    rpf(MFiles).
