/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: driver.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:44:31 CDT 1990
*   $Log:	driver.pro,v $
**   Revision 1.2  90/07/05  17:22:08  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:31:10  arun
**   Initial revision
**         */

/*
*    ENTRY POINTS 
*
*   TO DRIVE TESTS 
*      mixd(+GOAL)   -- display one residue
*      mixb(+GOAL)   -- perform mix_bag and display residue
*
*   TO DRIVE GENERAL PROGRAMS
*      peval(+GOAL, -RESIDUE)  -- Returns one Residue
*      peval_procedure(+GOAL, -RESIDUES) -- returns ALL RESIDUES
*
*   TO DRIVE SPECIALIZATION OF INTERPRETERS 
*     peval_int(+INT_GOAL, +META_ARG, +LANG_ATOMS, -RESIDUES)
*     mix_and_output(+INT_GOAL, +INT_ARG, +LANG_ATOMS)
*      

  /****   DRIVER TO RUN TESTS  ****/

%   mixd(+GOAL)
%       partially evaluates GOAL and displays ALL Residual clauses

  mixd(Goal) :-
     drive_mix(Goal, Clause),
     display_clauses([Clause]), 
     fail.
  mixd(_Goal).

   /* Test what "BAG" means */
  mixb(Goal) :-
     mix_bag(transparent, Goal, Residue),
     flatten_conjunction((Goal:-Residue), Cl),
     display_clauses([Cl]).


  /**** DRIVER FOR GENERIC GOALS ****/

%  peval(+GOAL, -RESIDUE)
%     partially evaluate GOAL and return one RESIDUE
%     Others are returned on backtracking

peval(PevalGoal, ResidueClause) :-
    drive_mix(PevalGoal, MixedClause),
    FlattenedClause = MixedClause,
    rewrite(FlattenedClause, ResidueClause).

%  peval_procedure(+PREDICATE, -RESIDUES)
%    generate all RESIDUES after parially evaluating PREDICATE

peval_procedure(PevalGoal, Head, Tail) :-
   findall(NewClause, peval(PevalGoal, NewClause), NewProc),
   make_diff_list(NewProc, Head-Tail).


  /****  DRIVER FOR SPECIALIZING INTERPRETERS  ****/

%  peval_int(+INT_GOAL, +INT_ARG, +LANG_ATOMS, -RESIDUES)
%     Partially evaluates INT_GOAL wrt all atoms in LANG_ATOMS
%     and return result in a list of RESIDUES. One entry INT_GOAL has
%     the variable INT_ARG at its meta-argument position. 


peval_int(IntGoal, IntArg, LangAtoms, Residues) :-
    peval_int(IntGoal, IntArg, LangAtoms, Residues, []).

peval_int(_IntGoal, _IntArg, [], Head, Head).
peval_int(IntGoal, IntArg, [LAtom|LangAtoms], Head, Tail) :-
   create_goal_to_peval(IntGoal, IntArg, LAtom, PevalGoal),
   peval_procedure(PevalGoal, Head, Tail1),
   peval_int(IntGoal, IntArg, LangAtoms, Tail1, Tail).

% create_goal_to_peval(+INT_GOAL, +INT_ARG, +LAtom, -PEVALGOAL)

   create_goal_to_peval(IntGoal, IntArg, LAtom, CopyIntGoal) :-
        copy_hook((IntArg,IntGoal), (LAtom,CopyIntGoal)).


%  mix_and_output(+INT_GOAL, +INT_ARG, +LANG_ATOMS)
%     same as peval_int/4 except it outputs the RESIDUES to the
%     standard ouput

mix_and_output(IntGoal, IntArg, LangAtoms) :-
    clean_rewrite_relation, 
    peval_int(IntGoal, IntArg, LangAtoms, NewProg),
    display_clauses(NewProg).



   /*   END OF DRIVER.PRO %%% */
