/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: meta_goal.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:44:38 CDT 1990
*   $Log:	meta_goal.pro,v $
**   Revision 1.2  90/07/05  17:22:19  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:31:12  arun
**   Initial revision
**         */

/* SUPPORT FOR TESTING, CREATING and MANIPULATING METAGOALS */

%  is_meta_goal(?GOAL)
%     tests if GOAL is a metagoal

  is_meta_goal(MetaGoal) :-
      meta_goal(_Flavor, MetaGoal, _MetaArg).

%  meta_goal(+FLAVOR, ?METAGOAL, ?METAARG)
%     test or get the METAGOAL and its METAARGument corresponding to a
%     flavor

  meta_goal(Flavor, MetaGoal, MetaArg) :-
      flavor(Flavor, MetaGoal),
      meta_goal(MetaGoal, MetaArg).

%  plug_in_object_goal(+METAGOAL, +GOAL)
%       Put GOAL as a meta-argument of METAGOAL

  plug_in_object_goal(ProcGoal, PevalGoal) :-
      is_meta_arg(ProcGoal, PevalGoal).

%  get_meta_argument(+METAGOAL, -GOAL)
%     get the GOAL at the meta-argument position of METAGOAL

  get_meta_argument(MetaGoal, ObjGoal) :-
      is_meta_arg(MetaGoal, ObjGoal).

% get_rest_arguments(+METAGOAL, -RESTARGS)
%    Get the list of RESTARGS in METAGOAL except the meta argument.
%    NOTE: Due to the use of delete_s the following may break if there
%    is another argument structurally equivalent to the meta-argument.

   get_rest_arguments(MetaGoal, RestArgs) :-
        is_meta_arg(MetaGoal, MetaArg),
        MetaGoal =.. [_|MetaArgs1],
        delete_s(MetaArgs1, MetaArg, RestArgs).

% get_goal_arguments(+GOAL, -ARGUMENTS)
%      get ARGUMENTS of the object GOAL

   get_goal_arguments(Goal, Args) :-
        Goal =.. [_Functor|Args].

%  is_meta_arg(+METAGOAL, -METAARG)
%      test or get the METAARGument of a METAGOAL

  is_meta_arg(MetaGoal, MetaArg) :-
      meta_goal(MetaGoal, MetaArg).


%  create_goal_template(+METAGOAL, -GMETAGOAL)
%      Given a METAGOAL it creates a template GMETAGOAL that has the
%      same function and arity as METAGOAL. At its meta-argument
%      positions it has a term with the same functor and arity as the
%      meta-argument of METAGOAL. All other arguments of the meta-goal
%      and its meta-argument are different and new variables

  create_goal_template(MetaGoal, GMetaGoal):-
      get_meta_argument(MetaGoal, ObjGoal),
      generalize_term(MetaGoal, GMetaGoal),
      generalize_term(ObjGoal, GObjGoal),
      plug_in_object_goal(GMetaGoal, GObjGoal).

   /*  END OF META_GOAL.PRO %%*/
