/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: rewrite.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:44:41 CDT 1990
*   $Log:	rewrite.pro,v $
**   Revision 1.2  90/07/05  17:22:02  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:31:12  arun
**   Initial revision
**         */

/* Modification history
   April 27, 1990:
      Added a variable "rewrite" with value on/off.
      Used to decide whether or not to perform rewriting
*/
 
/*
*  Supports rewriting of interpreter goals in meta-predicates
*  as well
*  The actual rewrite rule is separated from the text so that
*  and taken as a parameter
*/

% rewrite(+PREDICATE, -RE_PREDICATE)
%   translates the PREDICATE into RE_PREDICATE. All `predicates' that
%   arguments to the PREDICATE structure are translated recursively.

rewrite(Predicate, RPredicate) :-
  get_value(rewrite, on) 
   -> rewrite_x(Predicate, RPredicate)
   ; Predicate = RPredicate.

  rewrite_x(Var, RVar) :- 
     var(Var),!,
     rewrite_atom(Var, RVar).
  rewrite_x(MetaPred, RMetaPred) :-
    meta_property(_, MetaPred, Property),!,
    functor(MetaPred, Functor, Arity),
    functor(RMetaPred, Functor, Arity),
    rewrite_m(Arity, MetaPred, Property, RMetaPred).
  rewrite_x(Atom, RAtom) :-
    rewrite_atom(Atom, RAtom).

  rewrite_atom(Atom, RAtom) :-
    expand_predicate(Atom, RAtom),!.
  rewrite_atom(Atom, Atom).

  rewrite_m(0, _Pred, _Property, _RPred) :-!.
  rewrite_m(NthArg, Pred, Property, RPred) :-
	  arg(NthArg, Pred, A),
	  arg(NthArg, Property, P),
	  arg(NthArg, RPred, F),
	  N1 is NthArg -1,
	 (is_meta_argument(P)
	    -> rewrite_x(A, F)
	    ; F = A),
	 rewrite_m(N1, Pred, Property, RPred).


%  clean_rewrite_relation <-
%      removes the rewrite relations memo-ed in the database.


% help_param(?VARIABLE, -HELPMESSAGE, -TEST_FOR_VALUE, -DEFAULT)

   help_param(rewrite,
	[['The variable ''rewrite'' is a flag used to turn rewriting of'],
         ['goals on or off.'],
         [' '],
	 ['It can be set to'],
	 [' '],
	 [' on              -- Perform rewriting'],
	 [' off             -- Do not perform rewriting']
        ],

	 X^(nonvar(X),
             member(X,[off, on])
           ),

	 off           % DEFAULT TEST
       ).

/*  END OF REWRITE.PRO */
