/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: meta.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:37:13 CDT 1990
*   $Log:	meta.pro,v $
**   Revision 1.2  90/07/05  17:24:27  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:29:08  arun
**   Initial revision
**         */

/*    PARTIALLY EVALUATE META-PREDICATES call/1, ;/2, ->/3, not/1
*
*     No amount of documentation would aid in understanding this part.
*     I therefore leave you in silence. Comments would only clutter
*     the text and your thought.
*
*     But before i shut up: a tip for PORTING.
*     NOT (like God) has various names. Some systems refer to it by
*     not/1 and others by \+. To make porting easy I have abstracted
*     the structure from this code by using make_not/2,
*     (see promix.xref to locate where it is). I may probably move it
*     around while restructuring the code
*/

/*%  mix_meta(+GOAL, -RESIDUE)
*/

  mix_meta(call(X), Residue)  :- mix(X, Residue). /* % CALL %% */
  mix_meta(NotX, Residue)  :- make_not(_X, NotX), mix_meta_not(NotX, Residue).
  mix_meta((X->Y;Z), Residue) :- !, mix_meta_if((X->Y;Z), Residue).
  mix_meta((X;Z), Residue)    :- mix_meta_or((X;Z), Residue).

/*%%  If you are curious. This code can be unfolded and optimized. I
%%  like it like this because it
%%   1. allows one to cluster related code
%%   2. gives direct access to predicates when spy-pointing,
%%   3. should be possible to do the unfolding by self partial evaluation
*/

/*%%    NOT    %%%%
*/

/*%  mix_meta_not(+GOAL, -RESIDUE)
*/

  mix_meta_not(NotX, Residue) :-
     make_not(X, NotX),
     mix_bag_eval(X, MixX),!,
     mix_not(MixX, NotMixX),
     make_meta_residue(NotMixX, Residue).

     mix_not(true, _) :- !,fail.
     mix_not(fail, Residue):-!, make_atomic_residue(true, Residue).
     mix_not(Res, NotRes) :- make_not(Res, NotRes).

/*%%   IF-THEN-ELSE (->)  %%%%
*/

/*% mix_meta_if(+GOAL, -RESIDUE)
*/

  mix_meta_if((X->Y;Z), Residue) :-
      mix_bag_eval(X, MixX), 
      !,         /*  local cut after X */
      mix_if(MixX, Y, Z, Residue).

      mix_if(true, Then, _Else, Residue) :- 
	  !,
          mix_commit(Then, Residue).
      mix_if(fail, _Then, Else, Residue) :- 
	  !,
          mix_commit(Else, Residue).
      mix_if(MixX, Then, Else, MixXYZ) :-
	  mix_bag(Then, MixY),
	  mix_bag(Else, MixZ),
	  merge_ifs(MixX, MixY, MixZ, MixXYZ).

      merge_ifs(MixX, MixY, MixZ, Residue) :-
         make_meta_residue((MixX->MixY;MixZ), Residue).


/*      %  choose one of the two goals in mix_commit depending on how
      %  you like the residue for if when the condition is evaluable I
      %  like it split in many clauses.  Note a preceding side-effect
      %  or .. predicate would override this anyway.
*/
      mix_commit(X, MixX) :-  
/*	 %  mix_bag_eval_x(X, MixX).
*/
	 mix(X, MixX).

/*%%  DISJUNCTION   (;)  %%%
*/

/*% mix_meta_or(+GOAL, -RESIDUE)
*/

  mix_meta_or((X;Y), Residue) :-
      mix_bag_eval(X, MixX),
      mix_or(MixX, Y, Residue, CutOr),
      ( CutOr = cut_or,
	!
      ; true)
     ; mix_bag_eval_x(Y, Residue).

     mix_or(fail, _Y, _MixY, true) :- !, fail.
     mix_or(MixX, _Y, Residue, true) :-
          evaluable_residue(MixX),  /* but not fail */
          evaluate(MixX),
          !, make_atomic_residue(true, Residue).
     mix_or(MixX, Y, MixXorY, cut_or) :- 
	 mix_bag(Y, MixY),
	 merge_ors(MixX, MixY, MixXorY).

     merge_ors(MixX, MixY, Residue) :-
         make_meta_residue((MixX;MixY), Residue).

/*%%%  END OF META.PRO  %%%*/
