/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: residue.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:37:19 CDT 1990
*   $Log:	residue.pro,v $
**   Revision 1.2  90/07/05  17:24:30  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:29:09  arun
**   Initial revision
**         */

/*
*   The mix code is transparent to the structure of RESIDUE. This is
*   acheived by abstracting the structure, and performing all the
*   tests, creation, and manipulation of residue through routines
*   provided here.
*
*   The benefits of this approach are obvious and need not be
*   elaborated upon.
*
**  DATA ABSTRACTION FOR RESIDUE AND NOTES ON HANDLING OF CUT
*
*   A RESIDUE has one of the following structures
*
*          GOALS      : Prolog Goals
*
*  RESERVED STRUCTURES
*
*   The programs being partially evaluated should not contain the following
*   structures.
*      $$unifs/1    -- used to remember unifications generated internally
*
*  THIS VERSION DOES NOT HANDLE CUT.  THE STRUCTURE IS MAINTAINED AS
*  IN HERE TO MAINTAIN COMPATIBILITY WITH THE CUT HANDLING VERSION.

*/

/*%  make_atomic_residue(+GOAL, -RESIDUE)
%      create RESIDUE for atomic GOAL
*/

  make_atomic_residue(Goal, Goal).

/*%   make_meta_residue(+GOAL, -RESIDUE)
%      create RESIDUE for a meta GOAL
*/

  make_meta_residue(MetaRes, MetaRes).

/*% make_cut_residue(+GOALS, -WITH_CUT)
%   creates a WITH_CUT structure with GOALS as residue
*/

  make_cut_residue(!, !) :-
    writeln(['Warning ... ''!'' encountered ..']).

/*%  merge_ands(+RESIDUE1, +RESIDUE2, -RESIDUE)
%    creates RESIDUE12 representing a conjunction of RESIDUE1 and RESIDUE2
*/

  merge_ands(fail, _X, fail) :-!.
  merge_ands(MixX, MixY, MixXandY) :-
       flatten_conjunction((MixX,MixY), MixXandY).

/*%%%%  SUPPORT TO PROCESS BAG OF RESIDUES   %%%%%%%%%%%%
*/

/*% generate_unifications_for_bag(+SOLUTIONS, +VARS, -BAGS)
*/

   generate_unifications_for_bag([], _Vars, fail):-!.
   generate_unifications_for_bag([Sol], Vars, Res) :-!,
	 gx(Sol, Vars, Res).
   generate_unifications_for_bag([Sol|Us], Vars, Bag) :-
	 gx(Sol, Vars, Res),
	 generate_unifications_for_bag(Us, Vars, Ts),
         merge_ors(Res, Ts, Bag).

/*  % gx(+SOLUTION, +VARS, -RESIDUE)
*/

   gx(sol(Res,U), Vars, Residue) :-!,
       generate_unifications(Vars, U, Unifications),
       translate_to_unifs(Unifications, Unifs),
       merge_ands(Unifs, Res, Residue).
   gx(Sol,_,_) :-
/*      % A trap for errors during development 
*/
       writeln(['Error: Incorrect structure ', Sol,' to filter_residue_bag ']).


/*% generate_unifications(+VARS, +VALUES, -UNIFICATIONS)
%    if the VALUE corresponding to a VAR is bound it generates a
%    unification for it. If it a variable the two variables are
%    unified.
*/

   generate_unifications([], [], true).
   generate_unifications([Var|XVars], [CopyVar|CVars], TUnifs) :-
	var(CopyVar), !,
	Var = CopyVar,
	generate_unifications(XVars, CVars, TUnifs).
   generate_unifications([Var|XVars], [CopyVar|CVars], Unifs) :-
	generate_unifications(XVars, CVars, TUnifs),
	merge_conjunctions((Var = CopyVar), TUnifs, Unifs).

/*   % translate_to_unifs(+UNIFS, -UNIF_STRUCTURE)
*/
   translate_to_unifs(true, true):-!.
   translate_to_unifs(Unifications, '$$unifs'(Unifications)).

  '$$unifs'(X) :- X.   /*  Just in case you wish to execute $$unif/1 */

/*% evaluable_residue(+BAG)
%    Tests when a BAG has residue that is evaluable. The conditions
%    at present may be too strong. They may be relaxed after
%    gaining proper experience.
*/

  evaluable_residue(('$$unifs'(_U),true)).
  evaluable_residue('$$unifs'(_X)).
  evaluable_residue(true).
  evaluable_residue(fail).

/*% eval_unifs_if_determinate(+BAG, -RESIDUE)
%   evaluates a BAG if determinate. The BAG itself is RESIDUE
%   otherwise. 
*/

  eval_unifs_if_determinate('$$unifs'(Unifs), Residue) :-!,
       call_hook(Unifs),
       make_atomic_residue(true, Residue).
  eval_unifs_if_determinate(Bag, Bag).

/*% evaluable_bag(+BAG)
%    succeeds if all the residues in BAG are evaluable
*/

  evaluable_bag((R;Rs)) :- !,
	 evaluable_residue(R),
	 evaluable_bag(Rs).
  evaluable_bag(R) :- evaluable_residue(R).

/*% evaluable_first(+BAG)
%   succeeds if the first residue in a BAG is evaluable
%    This should be used in handling not/1 and condition part of
%    if-then-else. (It isn't at present)
*/

  evaluable_first((X;_Y)) :-!,
	 evaluable_residue(X).
  evaluable_first(X) :- evaluable_residue(X).

/*%%   END OF RESIDUE.PRO  %%%%*/
