/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: set.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:37:23 CDT 1990
*   $Log:	set.pro,v $
**   Revision 1.2  90/07/05  17:24:21  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:29:10  arun
**   Initial revision
**         */

/*  PARTIALLY EVALUATE SET PREDICATES: setof, bagof, findall
*
*   This code may be best understood by concentration.
*
*   Refer: The accompanying paper for understanding the details
*/

/*%  mix_set(+SETPRED, -RESIDUE)
%     evaluates SETPRED. If fully evaluable it computes the set
%     operation at compile time. Otherwise the RESIDUE consists of a
%     similar set operation on a partially evaluated goal.
*/

  mix_set(findall(X,Y,Z), Residue) :-
	 mix_bag(Y, MixY),
         evaluate_set(MixY, findall(X,MixY,Z), Residue).

  mix_set(bagof(X,V^Y,Z), Residue) :- !,
	 mix_set_x(Y, MixY, NewVars),
         evaluate_set(MixY, bagof(X,(V,NewVars)^MixY,Z), Residue).

  mix_set(bagof(X,Y,Z), Residue) :-
	 mix_set_x(Y, MixY, NewVars),
         evaluate_set(MixY, bagof(X,NewVars^MixY,Z), Residue).

  mix_set(setof(X,V^Y,Z), Residue) :-
	 mix_set_x(Y, MixY, NewVars),
         evaluate_set(MixY, setof(X,(V,NewVars)^MixY,Z), Residue).

  mix_set(setof(X,Y,Z), Residue) :-
	 mix_set_x(Y, MixY, NewVars),
         evaluate_set(MixY, setof(X,NewVars^MixY,Z), Residue).

      mix_set_x(Y, MixY, NewVars) :-
	 mix_bag(Y, MixY),
	 extra_variables(MixY, Y, NewVars).

/*     % evaluate_set(+MIX, +SETPRED, -RESIDUE)
*/
     evaluate_set(MixY, SetPred, Residue) :-
	    (evaluable_bag(MixY) ->
		SetPred,
		make_atomic_residue(true, Residue)
	    ; make_meta_residue(SetPred, Residue)).


/*%%   END OF SET.PRO  %%%
*/

