/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: meta.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:36:18 CDT 1990
*   $Log:	meta.pro,v $
**   Revision 1.3  90/07/05  17:22:41  arun
**   New Release
**   
**   Revision 1.2  89/04/17  17:23:19  arun
**   Added mechanism to detect loops
**   
**   Revision 1.1  89/04/16  01:25:30  arun
**   Initial revision
**         */

/*    PARTIALLY EVALUATE META-PREDICATES call/1, ;/2, ->/3, not/1
*
*     No amount of documentation would aid in understanding this part.
*     I therefore leave you in silence. Comments would only clutter
*     the text and your thought.
*
*     But before i shut up: a tip for PORTING.
*     NOT (like God) has various names. Some systems refer to it by
*     not/1 and others by \+. To make porting easy I have abstracted
*     the structure from this code by using make_not/2,
*     (see cross reference to locate where it is). I may probably move it
*     around while restructuring the code
*/

%  mix_meta(+GOAL, -RESIDUE, +ANCESTORS)

  mix_meta(call(X), Residue, Ancestors)  :- 
       mix_transparent(X, Residue, Ancestors).
  mix_meta(NotX, Residue, Ancestors)  :- 
       make_not(_X, NotX), 
       mix_meta_not(NotX, Residue, Ancestors).
  mix_meta((X->Y;Z), Residue, Ancestors) :- !,
       mix_meta_if((X->Y;Z), Residue, Ancestors).
  mix_meta((X;Z), Residue, Ancestors) :- 
       mix_meta_or((X;Z), Residue, Ancestors).

  %%  If you are curious. This code can be unfolded and optimized. I
  %%  like it like this because it
  %%   1. allows one to cluster related code
  %%   2. gives direct access to predicates when spy-pointing,
  %%   3. should be possible to do the unfolding by self partial evaluation


   /*    NOT    */

%  mix_meta_not(+GOAL, -RESIDUE, +ANCESTORS)

  mix_meta_not(NotX, Residue, Ancestors) :-
     make_not(X, NotX),
     mix_bag(opaque, X, MixX, Ancestors),!,
     eval_first_residue(MixX, EvalMixX),
     mix_not(EvalMixX, NotMixX),
     make_meta_residue(NotMixX, Residue).

     mix_not(true, _) :- !,fail.
     mix_not(fail, Residue):-!, make_atomic_residue(true, Residue).
     mix_not(Res, NotRes) :- make_not(Res, NotRes).

   /*   IF-THEN-ELSE (->)  */

% mix_meta_if(+GOAL, -RESIDUE, +ANCESTORS)

  mix_meta_if((X->Y;Z), Residue, Ancestors) :-
      mix_bag_eval(opaque, X, MixX, Ancestors), 
      !,         /*  local cut after X */
      mix_if(MixX, Y, Z, Residue, Ancestors).

      mix_if(true, Then, _Else, Residue, Ancestors) :- 
	  !,
          mix_commit(Then, Residue, Ancestors).
      mix_if(fail, _Then, Else, Residue, Ancestors) :- 
	  !,
          mix_commit(Else, Residue, Ancestors).
      mix_if(MixX, Then, Else, MixXYZ, Ancestors) :-
	  mix_bag(transparent, Then, MixY, Ancestors),
	  mix_bag(transparent, Else, MixZ, Ancestors),
	  merge_ifs(MixX, MixY, MixZ, MixXYZ).

      merge_ifs(MixX, MixY, MixZ, Residue) :-
	 (  has_cut(MixX)
	  ; has_cut(MixY)
	  ; has_cut(MixZ)
	 ),!,
	 make_cut_residue((MixX->MixY;MixZ), Residue).
      merge_ifs(MixX, MixY, MixZ, Residue) :-
         make_meta_residue((MixX->MixY;MixZ), Residue).


      %  choose one of the two goals in mix_commit depending on how
      %  you like the residue for if when the condition is evaluable I
      %  like it split in many clauses.  Note a preceding side-effect
      %  or .. predicate would override this anyway.
      mix_commit(X, MixX, Ancestors) :-  
	 mix_transparent(X, MixX, Ancestors).
	 %  mix_bag_eval_x(transparent, X, MixX, Ancestors).


   /*   DISJUNCTION   ;/2  */

% mix_meta_or(+GOAL, -RESIDUE, +ANCESTORS)

  mix_meta_or((X;Y), Residue, Ancestors) :-
      mix_bag_eval(transparent, X, MixX, Ancestors),
      mix_or(MixX, Y, Residue, CutOr, Ancestors),
      ( CutOr = cut_or,
	!
      ; true)
     ; mix_bag_eval_x(transparent, Y, Residue, Ancestors).

     mix_or(fail, _Y, _MixY, true, _Ancestors) :- !, fail.
     mix_or(MixX, _Y, Residue, true, _Ancestors) :-
          evaluable_residue(MixX),  /* but not fail */
          evaluate(MixX),
          !, make_atomic_residue(true, Residue).
     mix_or(MixX, _Y, MixX, cut_or, _Ancestors) :-
	 evaluable_cut(MixX, _Before, _After),!.
     mix_or(MixX, Y, MixXorY, cut_or, Ancestors) :- 
	 mix_bag(transparent, Y, MixY, Ancestors),
	 merge_ors(MixX, MixY, MixXorY).

     merge_ors(MixXC, MixYC, Residue) :-
	 ( has_cut(MixXC)
	 ; has_cut(MixYC)
	 ), !,
	 make_cut_residue((MixXC;MixYC), Residue).
     merge_ors(MixX, MixY, Residue) :-
         make_meta_residue((MixX;MixY), Residue).

/*  END OF META.PRO  */
