/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: set.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:36:29 CDT 1990
*   $Log:	set.pro,v $
**   Revision 1.3  90/07/05  17:22:50  arun
**   New Release
**   
**   Revision 1.2  89/04/17  17:23:24  arun
**   Added mechanism to detect loops
**   
**   Revision 1.1  89/04/16  01:25:32  arun
**   Initial revision
**         */

/*  PARTIALLY EVALUATE SET PREDICATES: setof, bagof, findall
*   No comments, this code may be best understood by concentration.
*/

%  mix_set(+SETPRED, -RESIDUE, +ANCESTOR)
%     evaluates SETPRED. If fully evaluable it computes the set
%     operation at compile time. Otherwise the RESIDUE consists of a
%     similar set operation on a partially evaluated goal.


  mix_set(findall(X,Y,Z), Residue, Ancestor) :-
	 mix_bag(opaque, Y, MixY, Ancestor),
         evaluate_set(MixY, findall(X,MixY,Z), Residue).

  mix_set(bagof(X,V^Y,Z), Residue, Ancestor) :- !,
	 mix_set_x(Y, MixY, NewVars, Ancestor),
         evaluate_set(MixY, bagof(X,(V,NewVars)^MixY,Z), Residue).

  mix_set(bagof(X,Y,Z), Residue, Ancestor) :-
	 mix_set_x(Y, MixY, NewVars, Ancestor),
         evaluate_set(MixY, bagof(X,NewVars^MixY,Z), Residue).

  mix_set(setof(X,V^Y,Z), Residue, Ancestor) :-
	 mix_set_x(Y, MixY, NewVars, Ancestor),
         evaluate_set(MixY, setof(X,(V,NewVars)^MixY,Z), Residue).

  mix_set(setof(X,Y,Z), Residue, Ancestor) :-
	 mix_set_x(Y, MixY, NewVars, Ancestor),
         evaluate_set(MixY, setof(X,NewVars^MixY,Z), Residue).

      mix_set_x(Y, MixY, NewVars, Ancestor) :-
	 mix_bag(opaque, Y, MixY, Ancestor),
	 extra_variables(MixY, Y, NewVars).

   % evaluate_set(+MIX, +SETPRED, -RESIDUE)

     evaluate_set(MixY, SetPred, Residue) :-
	    (evaluable_bag(MixY) ->
		SetPred,
		make_atomic_residue(true, Residue)
	    ; make_meta_residue(SetPred, Residue)).


/*   END OF SET.PRO  */

