/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: cprolog.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:45:11 CDT 1990
*   $Log:	cprolog.pro,v $
**   Revision 1.2  90/07/05  17:24:14  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:33:54  arun
**   Initial revision
**         */

/*
  MODIFICATIONS

  June 27, 1990: Added a definition for multifile
          and operator declaration for multifile and ';'

  July 4, 1990:  By Ronen Feldman (feldman@cs.cornell.edu)
                Marked as [Ronen] in the text

  July 5, 1990: By Arun Lakhotia 
               findall when defined using bagof needs existential
               quantification. It also returns an empty list for
               no solution
  July 6, 1990 By Ronen Feldman
               corrected is_interpreted/1 predicate. It was
               instantiating its argument thereby leading to single
               solutions in certain cases.
*/

:- current_op(X, Y, not), op(900, fx, dynamic).
:- current_op(X, Y, not), op(900, fx, multifile).

%% :- current_op(X, Y, ','), X1 is X-10, op(X1, Y, ';'). [Ronen]

dynamic(_X).

multifile(_).

/* To suppress errorrs for undefined predicates */
portray_clause(X) :- !, fail.

file_exists(File) :-
  nofileerrors,
  see(File),
  seen,!,
  fileerrors.
file_exists(File) :-
  fileerrors, fail.

make_not(X, not(X)) :-!.
make_not(X, \+(X)).

/*% see_if_file_exists(+Filename)
*/
see_if_file_exists(X) :-
      see(X).
/*   % else fail.
*/

member(X,[X|_]).
member(X,[_|Xs]) :- member(X,Xs).

% I have added this since cprolog does'nt have findall. [Ronen]
findall(X,Y,Z) :- bagof(X,Y^Y,Z), !. % Should be Y^Y -- [Arun]
findall(_,_,[]).                     % Empty list for null solution


append([X|Xs],Ys,[X|Zs]) :- append(Xs,Ys,Zs).
append([],Xs,Xs).

/*% ground/1 is predefined in CProlog
*/

writeln([]) :- !,
    nl.
writeln([X|Xs]) :-
    write(X),
    writeln(Xs).


consult_program(File) :- consult(File).
reconsult_program(File) :- reconsult(File).

consult_data(File) :- consult(File).
reconsult_data(File) :- reconsult(File).

unfold_data(A, B) :- clause(A, B).

/*%    is_compiled(+GOAL)
%    is_interpreted(+GOAL)
%    is_built_in(+GOAL).
*/

     is_compiled(Goal) :-
           fail.
     is_interpreted(Goal) :-
           copy_hook(Goal,Goal1),clause(Goal1, _).  % [Ronen] copy_hook
     is_built_in(Goal) :- % before it was sysp(X) [Ronen]
          copy_hook(Goal,Goal1),sysp(Goal1). % added copy_hook
%     predicate_property(Goal, built_in). Should be removed [Arun]

/*% tabs_hook(+NUM)
*/
   tabs_hook(X) :- tab(X).

/*% read_hook(+TERM)
*/
   read_hook(X) :- read(X).

/*% call_hook(+GOAL)
*/
   call_hook(clause(X,Y)) :-!, unfold_clause(X,Y).
   call_hook(Goal) :- call(Goal).

/*% lt_hook(+X, +Y)
*/
   lt_hook(X, Y) :- X < Y.

/*% gt_hook(+X, +Y)
*/
   gt_hook(X, Y) :- X > Y.

/*% name_hook(NAME, STRING) 
*/
   name_hook(Name, String) :- name(Name, String).

/*% copy_hook(+TERM1, -TERM2)
*/
   copy_hook(X,Y) :- asserta('#$$'(X)),retract('#$$'(Y)).


/*%% END OF CPROLOG.PRO %%*/


