/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: consult_data.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:45:57 CDT 1990
*   $Log:	consult_data.pro,v $
**   Revision 1.2  90/07/05  17:23:39  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:35:17  arun
**   Initial revision
**         */

/*
*    CHANGES
*      April 24, 1990 AL
*        Fixed a fault with is_user_defined/1. It was failing
*        when the predicate corresponding to its argument was defined
*        but the heads of none of its clause unified with the goal.
*
*       February 18, 1990  by AL
*    Incorporated changes to fix the problem with evaluating programs
*    in virtual database. Required renaming is_interpreted/1 predicate
*    to is_user_defined/1 and an introducing the procedure
*    is_primitive/1.
*    These two procedures override their own previous definition in
*    case a virtual database is selected.
*    WARNING
*       It is necessary that this file be consulted at the end so that
*    the new definitions are not overridden accidentally.
*/

consult_data(File) :-
    track_file(File, ActualFile),
    writeln(['Consulting data file ', ActualFile, '..']),
    readfile(ActualFile, Clauses),
    assert_clauses(Clauses, ActualFile),
    writeln(['.. done']).

reconsult_data(File) :-
    track_file(File, ActualFile),
    purge_data_from_file(ActualFile),
    writeln(['Reconsulting data file ', ActualFile, '..']),
    readfile(ActualFile, Clauses),
    assert_clauses(Clauses, ActualFile),
    writeln(['.. done']).

    assert_clauses([], _File).
    assert_clauses([(X:-Y)|Cls], File) :-
	assertz('$$program'(X, Y, File)),
	assert_clauses(Cls, File).

    purge_data_from_file(File) :-
	    retract('$$program'(_,_,File)),
	    fail
	;	true.

/*% clean_virtual_database
*/

clean_virtual_database :-
      purge_data_from_file(_X).
      

/*% call_hook(+GOAL)
%       Evaluate GOAL. Uses prolog_goal/1 interpreter to evaluate
%       the GOAL if it is in the virtual database.
*/
   call_hook(clause(X,Y)) :-!, unfold_data(X,Y).
   call_hook(X):- is_primitive(X),!, call(X).
   call_hook(Goal) :- prolog_goal(Goal).

/*%    is_user_defined(+GOAL)
%        A GOAL that is defined in the program database. Its definition
%        is available for processing.
*/

is_user_defined(Goal) :-
   functor(Goal, Functor, Arity),
   functor(GeneralGoal, Functor, Arity),
   '$$program'(GeneralGoal, _Y, _),!.

/*%  is_primitive(+GOAL)
%     A GOAL that is defined but whose definition is not accessible
%     for any kind of manipulation
%     With virtual databes a GOAL not in the virtual database is
%     considered primitive.
*/

is_primitive(Goal) :-
     is_built_in(Goal)
   ; is_interpreted(Goal)
   ; is_compiled(Goal).

/*  unfold_data(+HEAD, -BODY)
%     The program in the virtual database is data to ProMiX.
%     unfold_data/2 helps access the definition of these programs
*/

unfold_data(A, B) :-
    '$$program'(A, B, _).

