/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: unify.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:43:05 CDT 1990
*   $Log:	unify.pro,v $
Revision 1.1  90/07/05  17:25:24  arun
New Release

*/

  %  unify(?TERM1, TERM2)
  %       Unifies TERM1 and TERM2
  %       The predicate does not uses built-in =/2 for unification.
  %       Instead it reifies the unification process.
  %       This definition is useful for partial evaluation.
  %       In the event that non-ground terms cannot be partially
  %       evaluated this definition may be used to unify terms as much
  %       as possible.

     unify(Var1, Term2) :-
         var(Var1), !,
        % explicitly unify a variable
         Var1 = Term2.
     unify(Term1, Var2) :-
         var(Var2), !,
        % explicitly unify a variable
         Var2 = Term1.
     unify(Term1, Term2) :-
       functor(Term1, Functor, Arity),
       functor(Term2, Functor, Arity),
       unify_x(Arity, Term1, Term2).
   
     unify_x(0, _Term1, _Term2).
     unify_x(NthArg, Term1, Term2) :-
	   NthArg > 0,
	   arg(NthArg, Term1, A1),
	   arg(NthArg, Term2, A2),
	   unify(A1, A2),
	   N1 is NthArg -1,
	   unify_x(N1, Term1, Term2).

