/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: knowledge.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:40:38 CDT 1990
*   $Log:	knowledge.pro,v $
**   Revision 1.2  90/07/05  17:22:27  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:30:06  arun
**   Initial revision
**         */

%  executable(+PREDICATE)
%     succeeds if a system PREDICATE is executable.
%     Rules for commonly used predicates are coded. For predicates
%     that may be left-out - similar rules may be written and added

   executable(X) :- var(X),!,fail.

   executable(var(X)) :- nonvar(X).
   executable(nonvar(X)) :- nonvar(X).
   executable(atomic(X)) :- nonvar(X).

   executable(_ is A) :- ground(A).
   executable(A + B) :- ground(A + B).
   executable(A - B) :- ground(A - B).
   executable(A * B) :- ground(A * B).
   executable(A / B) :- ground(A / B).
   executable(A == B) :- ground(A == B).
   executable(A \== B) :- ground(A \== B).
   executable(A < B) :- ground(A < B).
   executable(A > B) :- ground(A > B).
   executable(A =< B) :- ground(A =< B).
   executable(A >= B) :- ground(A >= B).

   executable(A @< B) :- ground(A @< B).
   executable(A @> B) :- ground(A @> B).
   executable(A @=< B) :- ground(A @=< B).
   executable(A @>= B) :- ground(A @>= B).

   executable(_ = _).
   executable(functor(G,_,_)) :- nonvar(G).
   executable(functor(_,F,A)) :- ground(F),ground(A).
   executable(clause(A,_)) :- nonvar(A).
   executable((X =.._Y)) :- nonvar(X),!.
   executable((_X =..Y)) :- 
	  nonvar(Y), 
	  Y =..[F|_Fs],
	  nonvar(F).

   executable(true).
   executable(fail).

   executable(sys(X)) :- nonvar(X).
   executable(predicate_property(P,_)) :- nonvar(P).

%% evaluate(+Goal)
%%    execute Goal completely using underlying prolog engine
%%    rather than meta-interpreting it.
%   some Prolog systems give error on performing a clause/2
%   on a system goal. The following clauses traps such a goal
%   and fails it. Other goals are executed normally.

   evaluate(X) :- call_hook(X).

% immediately_evaluable(+GOAL)

immediately_evaluable(Goal) :-
	immediate(Goal).    /*      A hook for user defined goals. */
immediately_evaluable(Goal) :-
   executable(Goal).

%  is_side_effect(+PREDICATE)
%       Succeeds if PREDICATE generates side effect. Used for system
%       predicates. Propagation of effect is considered separately.

is_side_effect(Goal) :-
      side_effect(Goal).     /* A hook for user defined goals */
is_side_effect(Goal) :-
      sys_side_effect(Goal).

   sys_side_effect(X) :- var(X),!,fail.
   sys_side_effect(write(_)).
   sys_side_effect(nl).
   sys_side_effect(assert(_)).
   sys_side_effect(asserta(_)).
   sys_side_effect(assertz(_)).
   sys_side_effect(retract(_)).
   sys_side_effect(retracta(_)).
   sys_side_effect(retractz(_)).

%  is_extra_logical(+PREDICATE)
%       Succeeds if PREDICATE generates is extra_logical. Used for system
%       predicates. Propagation of effect is considered separately.

   is_extra_logical(Goal) :-
	extra_logical(Goal).  /* A hook for user defined goals */
   is_extra_logical(Goal) :-
	sys_extra_logical(Goal).


   sys_extra_logical(X) :- var(X),!,fail.
   sys_extra_logical(var(X)) :- var(X).
   sys_extra_logical(nonvar(X)) :-nonvar(X).

% is_control_predicate(+PREDICATE)
% is_set_predicate(+PREDICATE)

  is_control_predicate(Predicate) :-
       meta_property(meta, Predicate, _).

  is_set_predicate(Predicate) :-
       meta_property(set, Predicate, _).

   /*       END OF KNOWLEDGE.PRO %%% */
