/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: loop.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:40:41 CDT 1990
*  $Log:	loop.pro,v $
*   Revision 1.2  90/07/05  17:22:23  arun
*   New Release
*   
*   Revision 1.1  89/04/17  17:20:50  arun
*   Initial revision
*         */

/*
*   Modification:
*   July 5, 1990: Set the default value of "loop_check_type"
*        to "off".
*/

% computation_is_looping(+GOAL, +ANCESTORS)
%     succeeds if GOAL "belongs" to the list of ANCESTORS
%
%     The "belongs to" checks may be one of four tests given by
%     Frank van Harmelen (frank@aipna.ed.ac.uk) in his thesis
%     These are user selectable

   computation_is_looping(_Goal, _Ancestor) :-
      get_value(loop_check_type, off),!,
      fail.
   computation_is_looping(Goal, Ancestor) :-
      computation_is_looping_x(Goal, Ancestor).

      computation_is_looping_x(Goal, [Ans|_Ancestors]) :-
	     check_loop(Goal, Ans), !.
      computation_is_looping_x(Goal, [_Ans|Ancestors]) :-
	     computation_is_looping_x(Goal, Ancestors).

% check_loop(+GOAL, +ANS)
%   Loop determination checks for ProMiX

   check_loop(Goal, Ans) :-
      get_value(loop_check_type, Type),
      check_loop(Type, Goal, Ans).

   check_loop(unify, Goal, Ans) :-
      % Goal unifies with an Ancestor 
       unifiable(Goal, Ans).

   check_loop(instance, Goal, Ans) :-
      % Goal is more general than an Ancestor 
       more_general(Goal, Ans).

   check_loop(strict_instance, Goal, Ans) :-
      % Goal is a strict instance of an Ancestor 
       more_general(Goal, Ans),
       \+ more_general(Ans, Goal).

   check_loop(variant, Goal, Ans) :-
      % Goal is an alphabetic variant of an Ancestor 
       instance_of(Goal, Ans),
       instance_of(Ans, Goal).

   check_loop(less_general, Goal, Ans) :-
      % Goal is less general than Ancestor 
      less_general(Goal, Ans).

% help_param(?VARIABLE, -HELPMESSAGE, -TEST_FOR_VALUE, -DEFAULT)

   help_param(loop_check_type,
	[['The variable ''loop_check_type'' controls the type of test'],
	 ['performed to check if a GOAL *belongs* to the list of ANCESTORS'],
	 [' '],
	 ['The values it can be set to are'],
	 [' '],
	 [' unify             -- GOAL unifies with ANS'],
	 [' instance          -- GOAL is more general that ANS'],
	 [' strict_instance   -- GOAL is strictly more general than ANS'],
	 [' variant           -- GOAL is a symbolic variant of ANS'],
	 [' off               -- loop checking is turned off '],
         [' less_general      -- GOAL is less general than ANS (test purpose)'],
	 ['       where ANS is some goal in the ANCESTORS list']
	],

	 X^(nonvar(X),
             member(X,[off,unify,instance,strict_instance,variant,less_general])
           ),

	 off           % DEFAULT TEST
       ).

/* END OF LOOP.PRO */


