/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: allpred.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:37:36 CDT 1990
*   $Log:	allpred.pro,v $
**   Revision 1.2  90/07/05  17:23:17  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:23:04  arun
**   Initial revision
**         */

/*
*     Purpose: Mechanism to gather all predicates relevant for
*       any work. 
*/

/*%   preds_in_file(+FILE, -DIFF_PREDS_IN_PROGRAM)
%     gets a DIFF_PREDS_IN_PROGRAM defined in a FILE
%     readfile/2 takes care of immediate operands, and consult command
*/

  preds_in_file(File, PredsDiffList) :-
      track_file(File, F),
      readfile(F, Contents),!,
      preds_in_program(Contents, PredsDiffList).

/*%  all_user_preds(+PREDICATE, ?PRED_LIST)
%       returns a PRED_LIST of all user defined predicates
%       referred by the given PREDICATE.
*/

  all_user_preds(Goal,PredList) :-
     all_preds(Goal,List),
     filter_preds(List,X,(\+sys(X)),PredList).

/*%  all_preds(+PREDICATE, ?PRED_LIST)
%       returns a PRED_LIST of all the predicates referred by
%       the given PREDICATE
%       --  It returns an incomplete list
*/

  all_preds(Var,_InList) :-
    var(Var),!.
  all_preds(true,_InList):-!.
  all_preds((GoalA,GoalB),InList) :-!,
      all_preds(GoalA,InList),
      all_preds(GoalB,InList).
  all_preds((GoalA;GoalB),InList) :-!,
      all_preds(GoalA,InList),
      all_preds(GoalB,InList).
  all_preds(Pred,InList) :-
      is_meta_predicate(Pred),!,
      functor(Pred,F,N),
      check_and_add(F/N,InList,_),
      all_preds_meta_pred(Pred,InList).
  all_preds(Head,InList) :-
      functor(Head,F,N),
      check_and_add(F/N,InList,Check),
      (Check = previous ;
	/* if the predicate is added to the list now */
      functor(_Head,F,N),
      all_clauses(_Head,ClauseList),!,
      all_preds_clauses(ClauseList,InList)).
   all_preds(Goal,InList) :-
      /* system goal, or undefined goal */
      functor(Goal,F,N),
      check_and_add(F/N,InList,_).

      all_preds_clauses([],_InList).
      all_preds_clauses([(_H:-B)|RestCs],InList) :-
	 all_preds(B,InList),
	 all_preds_clauses(RestCs,InList).

  all_preds_meta_pred(\+X,InList) :-
     all_preds(X,InList).
  all_preds_meta_pred(call(X),InList) :-
     all_preds_meta_pred(X,InList).

  all_preds_meta_pred((Cond->Then;Else),InList) :-
      all_preds(Cond,InList),
      all_preds(Then,InList),
      all_preds(Else,InList).

  all_preds_meta_pred(if(Cond,Then,Else),InList) :-
      all_preds(Cond,InList),
      all_preds(Then,InList),
      all_preds(Else,InList).

  all_preds_meta_pred(call(Goal),InList) :-
     all_preds(Goal,InList).
  all_preds_meta_pred(findall(_Template,Generator,_List),InList) :-  
      all_preds(Generator,InList).
  all_preds_meta_pred(bagof(_Template,Generator,_Bag),InList) :-	
      all_preds(Generator,InList).
  all_preds_meta_pred(setof(_Template,Generator,_Set),InList) :-
      all_preds(Generator,InList).

/*%%   == SUPPORT PROCEDURES ===== */

  filter_preds(InList,_X,_Test,[]) :-
     var(InList),!.
  filter_preds([],_X,_Test,[]) :-!.
  filter_preds([F/N|InList], X, Test, [F/N|OutList]) :-
     functor(I,F,N),
     \+ \+ ( X = I, call_hook(Test)),!,
     filter_preds(InList,X,Test,OutList).
  filter_preds([_I|InList], X, Test, OutList) :-
     filter_preds(InList,X,Test,OutList).

/*%  preds_in_program(+CLAUSES, -PREDICATES)
%     Gives a difference list of PREDICATES defined in CLAUSES
*/

  preds_in_program(Clauses, Head-Tail) :-
      preds_in_program(Clauses, Head, Tail, Head).

  preds_in_program([], Tail, Tail, _AccList).
  preds_in_program([Clause|Clauses], Head, Tail, AccList) :-
      rule_head(Clause, Pred),
      generalize_term(Pred, GeneralPred),
      \+ member_x(GeneralPred, AccList), !,
      Head = [GeneralPred|NewHead],
      preds_in_program(Clauses, NewHead, Tail, AccList).
  preds_in_program([_|Clauses], Head, Tail, AccList) :-
      preds_in_program(Clauses, Head, Tail, AccList).

/*%%   END OF ALLPRED.PRO
*/

