/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: io.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:37:47 CDT 1990
*   $Log:	io.pro,v $
**   Revision 1.3  90/07/05  17:23:03  arun
**   New Release
**   
**   Revision 1.2  89/04/17  17:25:17  arun
**   Added write_multi_lines/1.
**   
**   Revision 1.1  89/04/16  01:23:06  arun
**   Initial revision
**         */
/*
*    Purpose: support predicates for i/o
*/

/*% display_clauses(+LIST_OF_CLAUSES)
%     Displays a LIST_OF_CLAUSES in a pretty printed
%     form
*/

  display_clauses([Cl|Cls]) :-
      copy_hook(Cl,Cl1),
      display_clause(Cl1),
      display_clauses_x(Cl1, Cls).
  display_clauses([]) :- nl.

  display_clauses_x(Previous, [Next| Rest]) :-
	  clause_head(Previous, PrevHead),
	  clause_head(Next, NextHead),

	  functor(PrevHead, PrevFunctor, PrevArity),
	  functor(NextHead, NextFunctor, NextArity),

	  (PrevFunctor/PrevArity = NextFunctor/NextArity; nl),!,
	  display_clauses([Next|Rest]).
  display_clauses_x(_Previous, Rest) :-
	  display_clauses(Rest).

/*% display_clause(+CLAUSE)
%     Display one CLAUSE in a pretty printed form
*/

  display_clause(Cl) :- portray_clause(Cl),!.
  display_clause(Cl) :- writeq_clause(Cl),!.
  display_clause(Cl) :- display_clause_symvar(Cl).

  writeq_clause(C1) :- writeq(C1), write(' .'), nl.

  display_clause_symvar(Cl) :-
      \+ \+ (symbolvars(Cl,0,_),     /* % \+\+ to undo unifications */
	   display_clause_symvar_xx(Cl)),
	   nl.

      display_clause_symvar_xx(Cl) :-
	  Cl =.. [(:-),Head,Body],
	  display_clause_symvar_x(Head,Body).

      display_clause_symvar_xx(Cl) :-
	  \+functor(Cl,(:-),_),
	  write(Cl),
	  write('.'),
	  nl.


      display_clause_symvar_x(Head, Body) :- 
	  and_to_list(Body,BodyList),
	  length(BodyList,N),
	  lt_hook(N, 2),
	  !,
	  write(Head),
	  write(' :- '),
	  write(Body),
	  write('.'),
	  nl.

      display_clause_symvar_x(Head, Body) :- 
	  write(Head),
	  write(' :- '),
	  nl,
	  and_to_list(Body,BodyList),
	  display_body(BodyList).

      display_body([G]) :- !,tabs_hook(6),write_pred(G),write('.'),nl.
      display_body([G|Gs]) :- tabs_hook(6),write_pred(G),write(','),nl,display_body(Gs).

      write_pred((X;Y)) :- !,
	 write('('), write(X), write(';'), write(Y), write(')').
      write_pred(X) :- write(X).

/*%  readfile(+FILE, -LIST)
%    reads FILE and returns a LIST of all its contents. If FILE
%    `consults' other file(s) using :- [...] command those files are
%    read too (recursively). LIST containts the content of all the
%    files.
*/

  readfile(Filename, Object) :-
    seeing(OldFilename),
    default_path(DirList),
    default_extension(ExtList),
    readfile(Filename,DirList,ExtList,Object),
    see(OldFilename).

  readfile(Filename,DirList,ExtList,Object) :-
    try_hard_to_see(Filename, DirList, ExtList),
    getObject(Object), !,
    seen.


    getObject(Object) :-
	read_hook(Term), \+ Term = end_of_file, !,
	getObject_(Term,Object).
    getObject([]).

      getObject_((:- X),Object) :-
	 immediate(X,IObj),
	 getObject(RObj),
	 append(IObj,RObj,Object).
      getObject_(Term,[Clause|Os]) :-    
	 term_to_clause(Term,Clause),
	 getObject(Os).

	 immediate((X1,X2),Ob) :- !,
	     immediate(X1,Ob1),
	     immediate(X2,Ob2),
	     append(Ob1,Ob2,Ob).
	 immediate([],[]) :-!.
	 immediate([X|Xs],Ob) :- !,
	     readfile(X,O),
	     immediate(Xs,Os),
	     append(O,Os,Ob).
	 immediate(X,[]) :-
	     call_hook(X).

/*% reconsult_data_files(+FILES)
%     loads a list of FILES as datas
%     translates the filenames for completing Home path
*/

reconsult_data_files([]) :-!.
reconsult_data_files([F|Fs]) :-!,
        reconsult_data_files(F),
        reconsult_data_files(Fs).
reconsult_data_files(F) :-
        track_file(F, Filename),
        reconsult_data(Filename).

/*% reconsult_program_files(+FILES)
%     loads a list of FILES as datas
%     translates the filenames for completing Home path
*/

reconsult_program_files([]) :-!.
reconsult_program_files([F|Fs]) :-!,
        reconsult_program_files(F),
        reconsult_program_files(Fs).
reconsult_program_files(F) :-
        track_file(F, Filename),
        reconsult_program(Filename).

/* some short hands */

rpf(F) :- reconsult_program_files(F).
rdf(F) :- reconsult_data_files(F).

/* write_multi_lines(LINES) 
*      writeln a list of LINES
*/

write_multi_lines([]).
write_multi_lines([L|Lines]) :-
      writeln(L),
      write_multi_lines(Lines).

/*%%  END OF IO.PRO  %%*/
