/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: list.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:37:50 CDT 1990
*   $Log:	list.pro,v $
**   Revision 1.3  90/07/05  17:23:07  arun
**   New Release
**   
**   Revision 1.2  89/04/16  02:24:18  arun
**   Added append_lists/2. 
**   Should generalize it to flatten_list/2 someday. Lets
**   see when the need arises.
**   
**   Revision 1.1  89/04/16  01:23:07  arun
**   Initial revision
**         */
/*
*   Purpose: Collect utilities related to operations
*      on Prolog list structure
*
*/

/*%  member_x(+TERM, +LIST)
%     tests if TERM is present in the "incomplete" LIST
*/

  member_x(_, Var) :-
      var(Var), !, fail.
  member_x(X, [X|_]).
  member_x(X, [_|Xs]) :-
      member_x(X, Xs).

/*%  check_and_now(+TERM, ?LIST, -STATUS)
%      Adds TERM to an *incomplete* LIST if it didnot contain
%        TERM before.
%      STATUS = previous -- LIST already had TERM
%      STATUS = now -- If TERM added NOW
*/

  check_and_add(X,L,now) :-
     var(L),!,
     L = [X|_].
  check_and_add(X,[X|_],Check) :-!,
     Check = previous.
  check_and_add(X,[_|L],Check) :-
     check_and_add(X,L,Check).

/*%  make_diff_list(+LIST, -DIFF_LIST)
%    Creates a DIFF_LIST with LIST@TAIL-TAIL
*/

  make_diff_list(Var, Var-Var) :-
	  var(Var),!.
  make_diff_list([], Tail-Tail).
  make_diff_list([X|Xs], [X|Zs]-Tail) :-
	  make_diff_list(Xs, Zs-Tail).

/*% split_list(-FIRST, -MIDDLE, -LAST, +LIST)
%       LIST = FIRST @ MIDDLE @ LAST
*/

  split_list(First, Middle, Last, List) :-
      split_list(Suffix, Last, List),
      split_list(First, Middle, Suffix).

/*% split_list(-SUFFIX, -REST, +LIST)
%    such that REST @ SUFFIX => LIST
*/

  split_list([], X, X).
  split_list([X|Xs], Y, [X|Z]) :-
	  split_list(Xs, Y, Z).

/*% append_lists(+LISTS, ONELIST)
%      appends a list of LISTS into ONELIST
*/

append_lists([X], X).
append_lists([X|Xs], Y) :-
    append_lists(Xs, Ys),
    append(X, Ys, Y).
      

/*%%% END OF LIST.PRO %%%*/
