/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: parameters.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:37:53 CDT 1990
*  $Log:	parameters.pro,v $
*   Revision 1.2  90/07/05  17:23:27  arun
*   New Release
*   
*   Revision 1.1  89/04/17  17:26:58  arun
*   Initial revision
*         */

/* Purpose: Support to save parameters of ProMiX with a standard
*    interface. Provides help on these variables and a
*    uniform way to setting and getting their values.
*/

:- dynamic '$promix_param'/2.

/* get_value(+VARIABLE, -VALUE)
*     gets the VALUE assigned to VARIABLE. If no value assigned
*     the default value set in help_param/4 is returned
*/

   get_value(Variable, Value) :-
       nonvar(Variable),
       '$promix_param'(Variable, Value),!.
   get_value(Variable, Value) :-
       help_param(Variable, _Message, _Test, Value).

/* set_value(+VARIABLE, +VALUE)
*     assigns VALUE to the VARIABLE. It tests for the validity of
*     VALUE before assigning it. If the validity is violated a help
*     message is printed to seek help.
*/

 set_value(Variable, Value) :-
       help_param(Variable, _Message, Value^Test, _),
       Test,
       (retract('$promix_param'(Variable, _));true),!,
       asserta('$promix_param'(Variable, Value)).
 set_value(Variable, Value) :-
     writeln(['The variable ', Variable, ' cannot take the value ', Value]),
     writeln(['Do ', help_value(Variable), ' to get help on its valid values']),
       fail.

/* set_value
*     Displays the current values for various variables in store
*/

  set_value :-
         writeln(['The current status of the various variables is ']),
	'$promix_param'(Variable, Value),
         writeln([Variable, '    --> ', Value]),
         fail.
  set_value.

/* help_value(+VARIABLE)
*      Displays a help message for the VARIABLE. The message is
*      contained in the predicate help_param/4. It should have
*      information about its usage and the values it can take
*/

  help_value(Variable) :-
       help_param(Variable, Message, _Test, Default),
       write_multi_lines(Message),
       nl,
       writeln(['Default Value : ', Default]),
       get_value(Variable, Value),
       writeln(['Currently Set to: ', Value]).
