/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: symbolvars.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:37:57 CDT 1990
*   $Log:	symbolvars.pro,v $
**   Revision 1.3  90/07/05  17:23:10  arun
**   New Release
**   
**   Revision 1.2  89/04/17  16:37:03  arun
**   For terms containing ground atom at the leaf position
**   symbolvars/3 had problem. A '!' had been missing.
**   Fixed it.
**   
**   Revision 1.1  89/04/16  01:23:08  arun
**   Initial revision
**         */
/*
*   Purpose: Generate unique alphabetic symbols for variables in a term
*   
*  NOTE: This code depends strongly on ASCII coding of characters
*        It will go kaput with EBCDIC machines. The ASCII dependent
*        code is localized in gen_symbolic_name/2. It may be recoded
*        for EBCDIC character set. (A trivial substitute is given in
*        src/port/ebcdic.pro
*/

/*% symbolvars(+TERM, +STARTN, -LASTN)
%    bind variables in TERM to symbols like A..Z, AA..ZZ
%    The symbol names are numbered 0 .. 25,26...
%*
%    STARTN corresponds to the start symbol to be used
%                for binding
%    LASTN-1 is the last symbol used for binding
*/

  symbolvars(Name, N, N1) :-
      var(Name),
      gen_symbolic_name(N,Name),
      N1 is N +1.
  symbolvars(Term, N1, N2) :-
      nonvar(Term),
      functor(Term, _Name, N), !,
      symbolvars(0,N,Term,N1,N2).
  symbolvars(Term,N1,N1) :-
     atom(Term).

  symbolvars(N,N,_Term,N1,N1).
  symbolvars(I,N,Term,N1,N3) :-
      lt_hook(I, N),
      I1 is I +1,
      arg(I1,Term,Arg),
      symbolvars(Arg,N1,N2),
      symbolvars(I1,N,Term,N2,N3).

/*      % gen_symbolic_name(+NUMBER, -SYMBOLICNAME).
      %     maps a NUMBER to its corresponding SYMBOLICNAME
      %     0 => A, 1 => B, .. ,26 => AA, 27 =>AB ...
      % NOTE: This code depends strongly on ASCII coding of characters
      %       It will go kaput with EBCDIC machines
      %       See src/port/ebcdic.pro for an alternative
*/

      gen_symbolic_name(Number,Name) :-
	 lt_hook(Number, 26),
	 Temp is 65 + Number,                    /* 5 = 'A' */
	 name_hook(Name, [Temp]),!.
      gen_symbolic_name(Number,Name) :-
/*	% generate the right-most character of the name
*/
	 Mod is Number mod 26,
	 gen_symbolic_name(Mod,LastChar),

/*	% generate the remaining name   
*/
	 Left is Number//26 -1,
	 gen_symbolic_name(Left,LeftName),

/*	% append the leftmost character to the remaining name
*/
	 append_atoms(LeftName,LastChar,Name).


/*%% END OF SYMBOLVARS.PRO %%*/
