/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: system.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:38:00 CDT 1990
*   $Log:	system.pro,v $
**   Revision 1.2  90/07/05  17:23:30  arun
**   New Release
**   
**   Revision 1.1  89/04/16  01:23:08  arun
**   Initial revision
**         */

/*  CHANGES
*     February 2, 1990 by AL
*   Redefined sys/1 in terms of is_primitive/1 and introduced new
*   definition of is_primitive/1 and is_user_defined/1. These
*   definitions suit well when the virtual database interface is not
*   used. Otherwise they are redefined in the file
*   port/virtual/consult_data.pro
*/

/*
*  Knowledge about system predicates
*/


/*% is_user_defined(+GOAL)
%       A GOAL whose definition is accessible for manipulation
%       Usually an interpreted goal in most implementations.
%       Redefined for virtual database interface.
*/
   is_user_defined(X) :- is_interpreted(X).

/*% is_primitive(+GOAL)
%       A GOAL whose definition is not accessible for further
%       manipulation. Usually a compiled or a built_in in most
%       implementations. Is redefined for virtual database.
*/
   is_primitive(X) :- 
            is_compiled(X)
          ; is_built_in(X).


/*% sys(+PREDICATE) 
%   PREDICATE is a system predicate.
%   All built_in and compiled predicates are considered system
%   predicates (since there definition cannot be accessed)
%   NOTE: To PORT to a system that does not support predicate_property/2
%   use the file port/sys.pro
*/

  sys(Goal) :- var(Goal), !, fail.
  sys(Goal) :-  is_primitive(Goal).


/*% is_meta_predicate(+PREDICATE)
%    Succeeds if PREDICATE is a meta_predicate
*/

  is_meta_predicate(Predicate) :- 
	meta_property(meta, Predicate, _Property).

/*%   meta_property(+TYPE, +PREDICATE, +PROPERTY)
%      if PREDICATE belongs to TYPE then
%      PROPERTY describes the property of its meta_arguments
%
%     PROPERTY = predicate(P1, P2, .., Pn))
%     where Pi is property of argument i
%     Pi may be '-' for non-meta argument
%                +  for meta-argument
*/

  meta_property(meta, (_;_),          (+;+)).
  meta_property(meta, ->(_,_),        ->(+,+)).
  meta_property(meta, if(_,_),        if(+,+)).
  meta_property(meta, if(_,_,_),      if(+,+,+)).
  meta_property(set,  findall(_,_,_), findall(-,+,-)).
  meta_property(set,  bagof(_,_,_),   bagof(-,+,-)).
  meta_property(set,  setof(_,_,_),   setof(-,+,-)).
  meta_property(meta, \+(_),          \+(+)).
  meta_property(meta, not(_),         not(+)).
  meta_property(meta, call(_),        call(+)).
  /*  caret used in setof/2 and bagof/2 */
  meta_property(none, ^(_,_),    ^(-,+)).
  /*  clause and conjunctions */
  meta_property(none, (_:-_),          (:-(+,+))).
  meta_property(none, (_,_),           (+,+)).

/*% is_meta_argument(+PROPERTY)
%      succeeds if PROPERTY indicates a meta-argument
%      used to test the property of arguments from information
%      returned by meta_property.
*/

  is_meta_argument(+).

/*%%%  END OF SYSTEM.PRO  %%%*/
