
/*%   file: Mec:Helper.Pl         
%   author: R.A.O'Keefe      
%   Updated: 12 October 82
%   Modified for TOPS-20	
%   Update author: Fernando Pereira
%   Update author: Arun Lakhotia, Dec 1986
%     Modification
%         Extended to replace a leading ~ by home(Home). [AL]
%         and added the trackfile predicate
%         Changed name/2 to name_hook/2 ..Feb 18, 89 [AL]
*/

/*%       try_hard_to_see(+FILENAME, +DEVICEDEFAULTS, +EXTENSIONDEFAULTS)
%           -- tries all the EXTENSION and DEVICE DEFAULTS (varying the
%           -- extensions first) until it succeeds in 'see'ing the file,
%           -- and fails if the file cannot be found.
%     -- DEVICEDEFAULTS == Default directories too 
*/
 
  try_hard_to_see(Title, DeviceDefaults, ExtensionDefaults) :-
	  nofileerrors,
	  expand_file(Title, DeviceDefaults, ExtensionDefaults, FullTitle),
	  see_if_file_exists(FullTitle), !,
	  fileerrors.
  try_hard_to_see(Title, _, _) :-
	  fileerrors,
	  write('** Can''t see '), writeq(Title), nl, fail.


  expand_file(Title, _DeviceDefaults, ExtensionDefaults, FullTitle) :-
	  expand_file_x(Title, _DeviceDefaults, ExtensionDefaults, FileName),
	  translate_filename(FileName, FullTitle).

  expand_file_x(Title, _DeviceDefaults, ExtensionDefaults, FullTitle) :-
	  attachExtn(Title, ExtensionDefaults, FullTitle).

  expand_file_x(Title, DeviceDefaults, ExtensionDefaults, FullTitle) :-
	  member(D,  DeviceDefaults),
	  name_hook(D, Ds), 
	  name_hook(Title, Ts),
	  append(Ds, Ts, Xs),
	  name_hook(T, Xs),
	  attachExtn(T, ExtensionDefaults, FullTitle).

  attachExtn(Title, _, Title).
  attachExtn(Title, ExtensionDefaults, FullTitle) :-
	  member(E, ExtensionDefaults),
	  name_hook(Title , Ts), 
	  name_hook(E, Es),
	  dot_char(Dot),
	  append(Ts, [Dot| Es], Fs),
	  name_hook(FullTitle, Fs).

  /*  in startup file, hence removed from here
  translate_filename(FileName, FullTitle) :-
          tilde_char(Tilde),
	  name_hook(FileName, [Tilde|RestString]),   % 126 = '~'
	  home(Home), 
	  (name_hook(Home, HomeStr); fatal_error(['Improper home definition :', Home])), !,
	  append(HomeStr, RestString, FullTitleStr),
	  name_hook(FullTitle, FullTitleStr).
  translate_filename(FileName, FileName).
  */

/*%  track_file(+PRIMARY, -FILE) 
%  track_file(+PRIMARY, +PATHS, +EXTENSIONS, -FILE) 
%     locates the first 'FILE' with 'PRIMARY' name in the
%     directory list 'PATH' and with extensions 'EXTLIST'
%       In any directory it first looks for a file with
%     no extension, then with the extension in sequence given
%     by 'default_extension/1'
%       It first looks for a file in the current directory
%     and then in directories in sequence given in consult_path/1
%
%     The extensions should *NOT* be specified with '.' ie [pl, c]
%     and the directories should be specified with the trailing '/'
%     example ['./', '/usr/local/lib/']
*/

  track_file(Primary, File) :-
    (default_path(Path); Path =[]),
    (default_extension(ExtList); ExtList=[]),!,
    track_file(Primary, Path, ExtList, File).

  track_file(Primary, Path, ExtList, File) :-
    expand_file(Primary, Path, ExtList, File),
    file_exists(File).
  track_file(Primary, _Path, _ExtList, _File) :-
     writeln(['Could not track file ', Primary]),
     !,fail.


/*%%  END OF TRYHARD.PRO %%%%*/
