/*  Copyright (C) 1989,90  Arun Lakhotia    */
/* File: startup.pro
*  Author: Arun Lakhotia
*  Released:  Thu Jul 5 16:44:06 CDT 1990
*   $Log:	startup.pro,v $
**   Revision 1.5  90/08/21  12:06:15  arun
**   Disabled the use of 'dynaload.pro' file. The use of virtual
**   database facility is mandatory when working with Quintus Prolog.
**   'dynaload.pro' file is no longer consulted.
**   
**   Revision 1.4  90/07/05  18:02:17  arun
**   *** empty log message ***
**   
**   Revision 1.3  90/07/05  17:01:26  arun
**   *** empty log message ***
**   
**   Revision 1.2  89/04/16  02:23:10  arun
**   Changed the startup prompt to take version number and generation date
**   from RCS generated information.
**   
**   Revision 1.1  89/04/16  01:43:23  arun
**   Initial revision
**         */
*/

/* %%
 %%    STARTUP FILE FOR ProMiX
 %%
 %%    ProMiX has a boot mechanism to bring the system up. This is the
 %%    first and only file to be loaded explicitly by the user. Other
 %%    system are automatically loaded on boot.
 %%
 %%    QUICK HINT
 %%       load this file, and query "boot"
 %%
 %%    For details see: Doc/Howto/STARTUP for documentation
 %%    Also see Doc/Howto/PORT for modifying parameters
*/

:- multifile(defsystem/3).

/*% home(-PATH)
%    PATH to ProMiX directory relative to current Directory
*/

     home('.').    /*    CAUTION: Should not be terminated by a '/' */

/*%  default_extension(-EXTENSIONS)
% list of EXTENSIONS to be used as default for searching files (both
%    program and data)
*/

default_extension(['pro']).

/*%%  FOR PORTING TO VARIOUS PROLOG IMPLEMENTATIONS
*/

/*% port_files(+SYSTEM, -COMMAND)
%    returns COMMAND to load files useful for porting to
%    a particular prolog SYSTEM
%  CAUTION: 
%      COMMAND is executed later, and is assumed to load
%      (consult/compile) the respective port files for SYSTEM
%      Substitution for HOME is not done to these files
*/

port_files(alspro,   compile('./src/port/alspro.pro')).
port_files(bim,      reconsult('./src/port/BIMprolog.pro')).
port_files(cprolog,  [-'./src/port/cprolog.pro',
                      -'./src/port/sys.pro']).
port_files(sicstus,  compile('./src/port/sicstus.pro')).
port_files(sbprolog, compile(['./src/port/sbprolog.pro',
                              './src/port/sys.pro'])).    
port_files(quintus,  compile(['./src/port/quintus.pro'])).
%                              './src/port/dynaload.pro'])).

/*% use_virtual_database
%     A flag to indicate use of virtual database for keeping
%     the programs that are data to ProMiX
%     When true the programs are asserted as facts, and the
%     clause/2 and other call_hook/1 predicates are translated so that
%     they clauses from virtual database.
*/

use_virtual_database.      % Use virtual database */
% use_virtual_database  :- fail.      % Don't use virtual database */

/*% tilde_char(-TILDE)
%  dot_char(-DOT)
%      numeric code for character TILDE and DOT
*/

tilde_char(Tilde) :- "~" = [Tilde].  /*  Hope this is not nonstandard */
dot_char(Dot)     :- "." = [Dot].

/*% default_path(+DIRECTORIES)
%      list of DIRECTORIES used for searching data files
*/

default_path([
                  './',
                  '~/examples/',
                  '~/examples/TEST/',
                  '~/examples/ndfa/',
                  '~/examples/udi/',
                  '~/examples/udi/data/',
                  '~/examples/metai/',
                  '~/examples/metai/data/',
                  '~/examples/dcg/',
                  '~/examples/dcg/data/',
		  '~/examples/inject/',
		  '~/examples/inject/acc/',
		  '~/examples/inject/context/',
		  '~/examples/inject/context/up/',
		  '~/examples/inject/context/down/'
                  ]).


/*%%   BOOTING THE SYSTEM
*/

/*% boot
%    Queries which SYSTEM to boot and boots
*/

boot :- 
        which_system(System),
        boot(System),
        nl, nl,
        signon.

/*% boot(+SYSTEM)
%    Prolog SYSTEM to boot for. Systems defined in port_files/2 are
%    supported
*/

boot(System) :-
        load_port_files(System),
        boot_programs(['~/src/defsystem.pro']),
        make_system(promix),!,
        load_vdb_interface.
boot(System) :-
        write(System), write(' not supported'), nl,
        write('See documentation on STARTUP and PORT'), nl.

/*%%     SUPPORT FOR LOADING PORT FILES
*/

/*% which_system(-SYSTEM)
%     Queries the name of Prolog SYSTEM to boot
*/

which_system(System) :-
       list_systems_supported,
       write('Startup for which system? '),
       read_system_name(System).

/*% load_port_files(+SYSTEM)
%     exectutes command to load port files for SYSTEM
%     and 'prolog_interpreter(SYSTEM)' for future reference
*/

load_port_files(System) :-
       port_files(System, LoadFiles),
       LoadFiles,
       abolish(prolog_interpreter, 1),
       asserta(prolog_interpreter(System)),!.
load_port_files(System) :-
       write('Failure in loading port files for the system '),
       write(System),nl,
       fail.

/*% list_systems_supported
%    Displays a list of Prolog SYSTEMS supported
*/

list_systems_supported :-
        nl, write('List of Prolog systems supported'), nl, 
        port_files(System, _Library),
	write(System), write('   '),
        fail.
list_systems_supported :- nl.

/*%  read_system_name(-SYSTEM)
%     Queries for SYSTEM name and verifies if it is supported.
%     Repeats query till a supported SYSTEM is selected
*/

read_system_name(System) :-
	read(System),
	port_files(System, _Files).
read_system_name(System) :-
	nl, write('Only one of the listed system is supported'), nl,
        write('The complete name should be typed terminated by a "." '),
        nl,write('Startup for which system? '),
        read_system_name(System).

/*%%  ****  GENERAL PURPOSE SUPPORT LIBRARY    *****
%%    Following code assumes that the port files are loaded
*/

/*%%   MAKE SYSTEMS
*/

/*%%  NOTE: In the following text the word SYSTEM means a user written software
%%        SYSTEM such as ProMiX. It should not be confused for the use
%%        of the same word in the context of Prolog systems used before
*/

/*% make_system(+SYSTEM)
%     load files belonging to a SYSTEM
%     if the system definitions is not loaded it is loaded using
%      load_system
*/

make_system(System) :-
      (defsystem(System, Directory, Sys);
       writeln([System, ' defsystem not loaded ...']), fail),!,
      make_system(Directory, Sys).

make_system(_Directory, []).
make_system(Directory, [F|Files]) :-
        load_system_unit(Directory, F),
	make_system(Directory, Files).

/*% load_vdb_interface
%    Load Virtual Database Support for Data Programs 
*/

load_vdb_interface :-
        use_virtual_database,!,
        nl, write('Using Virtual Database ...'), nl, nl,
        load_system_unit(_SysDir, module(virtual)).
load_vdb_interface.

load_system_unit(_SysDir, preloaded(_Files)) :-!.
load_system_unit(_SysDir, module(Module)) :-!,
      (defmodule(Module, ModDir, ModFiles)
      ; writeln([Module, ' defmodule not loaded ...']), fail),!,
      make_system(ModDir, ModFiles).
load_system_unit(Directory, F) :-
	append_atoms(Directory, F, FileName),
        translate_filename(FileName, ExpandedName),
        append_extension(ExpandedName, FullName),
	reconsult_program(FullName).

/*% boot_programs(+FILES)
%     loads a list of FILES as programs
%     translates the filenames for completing Home path
*/

boot_programs([]).
boot_programs([F|Fs]) :-
        translate_filename(F, FullF),
        reconsult_program(FullF),
        boot_programs(Fs).

/*%%   GENERAL UTILITIES
*/

/*%  translate_filename(+FILENAME, -FULLNAME)
%       If FILENAME starts with "~" it replaces it with path to home
%       to give FULLNAME
*/

translate_filename(FileName, FullTitle) :-
        tilde_char(Tilde),          /*  "~" */
        name_hook(FileName, [Tilde|RestString]),
        home(Home), 
       (name_hook(Home, HomeStr)
        ; fatal_error(['in startup : Improper home definition :', Home])),
       !,
	append(HomeStr, RestString, FullTitleStr),

	name_hook(FullTitle, FullTitleStr).
translate_filename(FileName, FileName).

/*% append_extension(+NAME, -NAME_WITH_EXT)
%   attaches extension to a file name
*/

append_extension(Name, FullName) :-
      default_extension([Ext|_]), !,
      append_atoms('.', Ext, ExtPart),
      append_atoms(Name, ExtPart, FullName).
append_extension(Name, Name).

/*% append_atoms(+ATOM1, +ATOM2, -ATOM12)
%       concatenates ATOM1 and ATOM2 to give ATOM12
*/

append_atoms(Atom1, Atom2, Atom12) :-
	name_hook(Atom1, A1s),
	name_hook(Atom2, A2s),
	append(A1s, A2s, A12s),
	name_hook(Atom12, A12s).

/*%  fatal_error(+MESSAGE)
%       displays Fatal Error MESSAGE and aborts
*/

fatal_error(Message) :-
   writeln(['** Fatal Error: '|Message]),
   abort.

rcs_version("$Revision: 1.5 $").
rcs_date("$Date: 90/08/21 12:06:15 $").

promix_version(ReleaseMessage) :-
      rcs_version(RCSVersion),
      strip_rcs_info(RCSVersion, Version),
      rcs_date(RCSDate),
      strip_rcs_info(RCSDate, Date),
      append_lists([Version, ": Released ", Date, " EST"], 
                RelString),
      name(ReleaseMessage, RelString).

strip_rcs_info(RCSMessage, Info) :-
    strip_rcs_info(Info, RCSMessage, []).

   strip_rcs_info(L) --> "$",!, strip_rcs_info2(L).
   strip_rcs_info(L) --> [_X], strip_rcs_info(L).

   strip_rcs_info2(L) --> " ",!, strip_rcs_info3(L).
   strip_rcs_info2(L) --> [_X], strip_rcs_info2(L).

   strip_rcs_info3([]) --> " $", !.
   strip_rcs_info3([X|Xs]) --> [X], strip_rcs_info3(Xs).

promix_copyright('Copyright (C) 1989,90, Arun Lakhotia').
promix_info('All rights reserved.').

virtual_database_message :-
    use_virtual_database, !,
    nl,
    write('Use of virtual database is selected. Use rdf/1 to load the'), nl,
    write('interpreter rpf/1 to load its meta_knowledge.'), nl.
virtual_database_message.

signon :-
   write('ProMiX '),
   promix_version(Version), write(Version), nl,
   promix_copyright(Message),
   write(Message), nl,
   promix_info(Info), write(Info), nl,
   virtual_database_message.

/*%%  END OF STARTUP */
